#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import cmind as cm
import os
import configparser


def preprocess(i):

    os_info = i['os_info']
    env = i['env']

    # Initialize ConfigParser
    config = configparser.ConfigParser()

    if env.get('CM_MLPERF_POWER_SERVER_CONF_FILE', '') != '':
        server_config_file = env['CM_MLPERF_POWER_SERVER_CONF_FILE']
    else:
        server_config_file = os.path.join(
            env.get('CM_MLPERF_POWER_SOURCE', ''),
            'ptd_client_server',
            'server.template.conf'
        )

    # Read the configuration file with error handling
    if not os.path.exists(server_config_file):
        raise FileNotFoundError(
            f"Server config file not found: {server_config_file}")

    config.read(server_config_file)
    # Update the server section
    try:
        config['server']['ntpServer'] = env['CM_MLPERF_POWER_NTP_SERVER']
        config['server']['listen'] = f"{env['CM_MLPERF_POWER_SERVER_ADDRESS']} {env['CM_MLPERF_POWER_SERVER_PORT']}"
    except KeyError as e:
        raise KeyError(f"Missing required environment variable: {e}")

    # Define number of analyzers and network port start
    num_analyzers = int(env.get('CM_MLPERF_POWER_NUM_ANALYZERS', 1))
    network_port_start = int(
        env.get(
            'CM_MLPERF_POWER_NETWORK_PORT_START',
            8888))

    # Ensure 'ptd' section exists
    if 'ptd' not in config:
        config.add_section('ptd')

    config['ptd']['ptd'] = str(env.get('CM_MLPERF_PTD_PATH', ''))
    config['ptd']['analyzercount'] = str(num_analyzers)

    # Add analyzers to the configuration
    for aid in range(1, num_analyzers + 1):
        analyzer_section = f'analyzer{aid}'
        if analyzer_section not in config:
            config.add_section(analyzer_section)

            # Add the analyzer subsection as keys under the 'ptd' section
            config[f'{analyzer_section}']['interfaceFlag'] = str(
                env.get('CM_MLPERF_POWER_INTERFACE_FLAG', ''))
            config[f'{analyzer_section}']['deviceType'] = str(
                env.get('CM_MLPERF_POWER_DEVICE_TYPE', ''))
            config[f'{analyzer_section}']['devicePort'] = str(
                env.get('CM_MLPERF_POWER_DEVICE_PORT', ''))
            config[f'{analyzer_section}']['networkPort'] = str(
                network_port_start + aid - 1)

    with open('tmp-power-server.conf', 'w') as configfile:
        config.write(configfile)

    print({section: dict(config[section]) for section in config.sections()})

    if env['CM_HOST_OS_TYPE'] == "windows":
        cmd_prefix = ""
    else:
        cmd_prefix = "sudo "

    cmd = env['CM_PYTHON_BIN_WITH_PATH'] + ' ' + os.path.join(
        env['CM_MLPERF_POWER_SOURCE'],
        'ptd_client_server',
        'server.py') + ' -c tmp-power-server.conf'
    if env.get('CM_MLPERF_POWER_SERVER_USE_SCREEN', 'no') == 'yes':
        cmd = cmd_prefix + ' screen -d -m ' + cmd + ' '
    else:
        cmd = cmd_prefix + cmd

    env['RUN_CMD'] = cmd

    return {'return': 0}


def postprocess(i):
    return {'return': 0}



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import os


def preprocess(i):

    os_info = i['os_info']

    env = i['env']

    recursion_spaces = i['recursion_spaces']

    file_name_c = 'clang.exe' if os_info['platform'] == 'windows' else 'clang'

    env['FILE_NAME_C'] = file_name_c

    if 'CM_LLVM_CLANG_BIN_WITH_PATH' not in env:
        r = i['automation'].find_artifact({'file_name': file_name_c,
                                           'env': env,
                                           'os_info': os_info,
                                           'default_path_env_key': 'PATH',
                                           'detect_version': True,
                                           'env_path_key': 'CM_LLVM_CLANG_BIN_WITH_PATH',
                                           'run_script_input': i['run_script_input'],
                                           'recursion_spaces': recursion_spaces})
        if r['return'] > 0:
            if r['return'] == 16:
                env['CM_REQUIRE_INSTALL'] = "yes"
                return {'return': 0}
            else:
                return r

    return {'return': 0}


def detect_version(i):

    r = i['automation'].parse_version({'match_text': r'clang version\s*([\d.]+)',
                                       'group_number': 1,
                                       'env_key': 'CM_LLVM_CLANG_VERSION',
                                       'which_env': i['env']})
    if r['return'] > 0:
        return r

    version = r['version']

    print(i['recursion_spaces'] + '    Detected version: {}'.format(version))

    return {'return': 0, 'version': version}


def postprocess(i):

    env = i['env']
    r = detect_version(i)
    if r['return'] > 0:
        return r

    version = env['CM_LLVM_CLANG_VERSION']
    env['CM_LLVM_CLANG_CACHE_TAGS'] = 'version-' + version
    env['CM_COMPILER_CACHE_TAGS'] = 'version-' + version + ',family-llvm'
    env['CM_COMPILER_FAMILY'] = 'LLVM'
    env['CM_COMPILER_VERSION'] = env['CM_LLVM_CLANG_VERSION']

    found_file_path = env['CM_LLVM_CLANG_BIN_WITH_PATH']

    found_path = os.path.dirname(found_file_path)

    file_name_c = os.path.basename(found_file_path)
    file_name_cpp = file_name_c.replace("clang", "clang++")

    env['CM_LLVM_CLANG_BIN'] = file_name_c

    # General compiler for general program compilation
    env['CM_C_COMPILER_BIN'] = file_name_c
    env['CM_C_COMPILER_WITH_PATH'] = found_file_path
    env['CM_C_COMPILER_FLAG_OUTPUT'] = '-o '
    env['CM_C_COMPILER_FLAG_VERSION'] = '--version'
    env['CM_C_COMPILER_FLAG_INCLUDE'] = '-I'

    env['CM_CXX_COMPILER_BIN'] = file_name_cpp
    env['CM_CXX_COMPILER_WITH_PATH'] = os.path.join(found_path, file_name_cpp)
    env['CM_CXX_COMPILER_FLAG_OUTPUT'] = '-o '
    env['CM_CXX_COMPILER_FLAG_VERSION'] = '--version'
    env['CM_CXX_COMPILER_FLAG_INCLUDE'] = '-I'

    env['CM_COMPILER_FLAGS_FAST'] = "-O4"
    # "-flto" - this flag is not always available (requires LLVMgold.so)
    env['CM_LINKER_FLAGS_FAST'] = "-O4"
    env['CM_COMPILER_FLAGS_DEBUG'] = "-O0"
    env['CM_LINKER_FLAGS_DEBUG'] = "-O0"
    env['CM_COMPILER_FLAGS_DEFAULT'] = "-O2"
    env['CM_LINKER_FLAGS_DEFAULT'] = "-O2"

    env['CM_GET_DEPENDENT_CACHED_PATH'] = env['CM_LLVM_CLANG_BIN_WITH_PATH']

    return {'return': 0, 'version': version}



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import os


def preprocess(i):

    os_info = i['os_info']

    env = i['env']

    cmd_string = ''

    # pre_run_cmds = env.get('CM_SSH_PRE_RUN_CMDS', ['source $HOME/cm/bin/activate'])
    pre_run_cmds = env.get('CM_SSH_PRE_RUN_CMDS', [])

    run_cmds = env.get('CM_SSH_RUN_COMMANDS', [])

    run_cmds = pre_run_cmds + run_cmds

    for i, cmd in enumerate(run_cmds):
        if 'cm ' in cmd:
            # cmd=cmd.replace(":", "=")
            cmd = cmd.replace(";;", ",")
            run_cmds[i] = cmd

    cmd_string += " ; ".join(run_cmds)
    user = env.get('CM_SSH_USER')
    password = env.get('CM_SSH_PASSWORD', None)
    host = env.get('CM_SSH_HOST')
    if password:
        password_string = " -p " + password
    else:
        password_string = ""
    cmd_extra = ''

    if env.get("CM_SSH_SKIP_HOST_VERIFY"):
        cmd_extra += " -o StrictHostKeyChecking=no"
    if env.get("CM_SSH_KEY_FILE"):
        cmd_extra += " -i " + env.get("CM_SSH_KEY_FILE")

    ssh_command = "ssh " + user + "@" + host + \
        password_string + cmd_extra + " '" + cmd_string + "'"
    env['CM_SSH_CMD'] = ssh_command

    return {'return': 0}


def postprocess(i):

    return {'return': 0}



#!/usr/bin/env python3

import os
import json
import numpy as np
from PIL import Image
import torch
import torchvision

SUPPORTED_EXTENSIONS = ['jpeg', 'jpg', 'gif', 'png']


def load_image(image_path, target_size, data_type='uint8', convert_to_bgr=False,
               normalize_data=False, normalize_lower=-1, normalize_upper=1,
               subtract_mean=False, given_channel_means='', given_channel_stds='',
               quantize=0, quant_scale=1, quant_offset=0, convert_to_unsigned=0):
    if not convert_to_bgr:
        image = Image.open(image_path).convert('RGB')
    else:
        image = Image.open(image_path).convert('BGR')

    tensor_image = torchvision.transforms.functional.to_tensor(image)
    mean = torch.as_tensor(given_channel_means)
    std = torch.as_tensor(given_channel_stds)
    normalized_image = (
        tensor_image - mean[:, None, None]) / std[:, None, None]

    resized_image = torch.nn.functional.interpolate(normalized_image[None],
                                                    size=(target_size,
                                                          target_size),
                                                    mode='bilinear')[0].numpy()

    if quantize == 1:
        resized_image = quantize_to_uint8(
            resized_image, quant_scale, quant_offset)

    original_height, original_width, _ = resized_image.shape
    batch_shape = (1, target_size, target_size, 3)
    batch_data = resized_image.reshape(batch_shape)

    return batch_data, original_width, original_height


def quantize_to_uint8(image, scale, offset):
    quantized_image = (
        image.astype(
            np.float64) /
        scale +
        offset).astype(
            np.float64)
    output = np.round(quantized_image)
    output = np.clip(output, 0, 255)
    return output.astype(np.uint8)


def preprocess_files(selected_filenames, source_dir, destination_dir, square_side,
                     data_type, convert_to_bgr, normalize_data, normalize_lower,
                     normalize_upper, subtract_mean, given_channel_means,
                     given_channel_stds, quantize, quant_scale, quant_offset,
                     convert_to_unsigned, new_file_extension):
    output_signatures = []

    for current_idx, input_filename in enumerate(selected_filenames):
        full_input_path = os.path.join(source_dir, input_filename)
        image_data, original_width, original_height = load_image(
            image_path=full_input_path,
            target_size=square_side,
            data_type=data_type,
            convert_to_bgr=convert_to_bgr,
            normalize_data=normalize_data,
            normalize_lower=normalize_lower,
            normalize_upper=normalize_upper,
            subtract_mean=subtract_mean,
            given_channel_means=given_channel_means,
            given_channel_stds=given_channel_stds,
            quantize=quantize,
            quant_scale=quant_scale,
            quant_offset=quant_offset,
            convert_to_unsigned=convert_to_unsigned
        )

        output_filename = f"{input_filename.rsplit('.', 1)[0]}.{new_file_extension}" if new_file_extension else input_filename
        full_output_path = os.path.join(destination_dir, output_filename)
        image_data.tofile(full_output_path)

        print(f"[{current_idx+1}]:  Stored {full_output_path}")
        output_signatures.append(
            f'{output_filename};{original_width};{original_height}')

    return output_signatures


def preprocess():
    source_directory = os.environ['CM_DATASET_PATH']
    destination_directory = os.environ['CM_DATASET_PREPROCESSED_PATH']

    intermediate_data_type = os.environ.get(
        'CM_DATASET_INTERMEDIATE_DATA_TYPE', np.float32)
    square_side = int(os.environ['CM_DATASET_INPUT_SQUARE_SIDE'])
    crop_percentage = float(os.environ['CM_DATASET_CROP_FACTOR'])
    inter_size = int(os.getenv('CM_DATASET_INTERMEDIATE_SIZE', 0))
    convert_to_bgr = int(os.getenv('CM_DATASET_CONVERT_TO_BGR', 0))
    offset = int(os.getenv('CM_DATASET_SUBSET_OFFSET', 0))
    volume = int(os.environ['CM_DATASET_SIZE'])
    fof_name = os.getenv('CM_DATASET_SUBSET_FOF', 'files.txt')
    data_type = os.getenv('CM_DATASET_DATA_TYPE_INPUT', 'float32')
    input_data_type = os.getenv('CM_DATASET_DATA_TYPE_INPUT', 'float32')
    data_layout = os.getenv('CM_DATASET_DATA_LAYOUT', '').lower()
    new_file_extension = os.getenv('CM_DATASET_PREPROCESSED_EXTENSION', '')
    normalize_data = int(os.getenv('CM_DATASET_NORMALIZE_DATA', '0'))
    subtract_mean = int(os.getenv('CM_DATASET_SUBTRACT_MEANS', '0'))
    given_channel_means = os.getenv('CM_DATASET_GIVEN_CHANNEL_MEANS', '')
    given_channel_stds = os.getenv('CM_DATASET_GIVEN_CHANNEL_STDS', '')
    quant_scale = float(os.environ['CM_DATASET_QUANT_SCALE'])
    quant_offset = float(os.environ['CM_DATASET_QUANT_OFFSET'])
    quantize = int(os.environ['CM_DATASET_QUANTIZE'])  # 1 for quantize to int8
    convert_to_unsigned = int(
        os.environ['CM_DATASET_CONVERT_TO_UNSIGNED'])  # 1 for int8 to uint8

    images_list = os.getenv('CM_DATASET_IMAGES_LIST')
    interpolation_method = os.getenv('CM_DATASET_INTERPOLATION_METHOD', '')

    annotations_filepath = os.environ['CM_DATASET_ANNOTATIONS_FILE_PATH']
    is_calibration = os.environ['CM_DATASET_TYPE'] == "calibration"
    image_file = os.getenv('CM_IMAGE_FILE', '')

    normalize_lower = float(os.getenv('CM_DATASET_NORMALIZE_LOWER', -1.0))
    normalize_upper = float(os.getenv('CM_DATASET_NORMALIZE_UPPER', 1.0))

    if given_channel_means:
        given_channel_means = np.fromstring(
            given_channel_means,
            dtype=np.float32,
            sep=' ').astype(intermediate_data_type)
        if convert_to_bgr:
            given_channel_means = given_channel_means[::-1]

    given_channel_stds = os.getenv('CM_DATASET_GIVEN_CHANNEL_STDS', '')
    if given_channel_stds:
        given_channel_stds = np.fromstring(
            given_channel_stds,
            dtype=np.float32,
            sep=' ').astype(intermediate_data_type)
        if convert_to_bgr:
            given_channel_stds = given_channel_stds[::-1]

    print(f"From: {source_directory}, To: {destination_directory}, Size: {square_side}, Crop: {crop_percentage}, InterSize: {inter_size}, 2BGR: {convert_to_bgr}, " +
          f"OFF: {offset}, VOL: '{volume}', FOF: {fof_name}, DTYPE: {data_type}, DLAYOUT: {data_layout}, EXT: {new_file_extension}, " +
          f"NORM: {normalize_data}, SMEAN: {subtract_mean}, GCM: {given_channel_means}, GSTD: {given_channel_stds}, QUANTIZE: {quantize}, QUANT_SCALE: {quant_scale}, " +
          f"QUANT_OFFSET: {quant_offset}, CONV_UNSIGNED: {convert_to_unsigned}, INTER: {interpolation_method}")

    if image_file:
        source_directory = os.path.dirname(image_file)
        selected_filenames = [os.path.basename(image_file)]
    else:
        if annotations_filepath and not is_calibration:
            with open(annotations_filepath, "r") as annotations_fh:
                annotations_struct = json.load(annotations_fh)
            ordered_filenames = [image_entry['file_name']
                                 for image_entry in annotations_struct['images']]
        elif os.path.isdir(source_directory):
            ordered_filenames = [
                filename for filename in sorted(
                    os.listdir(source_directory)) if any(
                    filename.lower().endswith(extension) for extension in SUPPORTED_EXTENSIONS)]
        else:
            raise FileNotFoundError(
                errno.ENOENT, os.strerror(
                    errno.ENOENT), source_directory)

        total_volume = len(ordered_filenames)

        if offset < 0:
            offset += total_volume

        if not volume:
            volume = total_volume - offset

        selected_filenames = ordered_filenames[offset:offset + volume]

    output_signatures = preprocess_files(selected_filenames, source_directory, destination_directory, square_side, data_type,
                                         convert_to_bgr, normalize_data, normalize_lower, normalize_upper,
                                         subtract_mean, given_channel_means, given_channel_stds, quantize,
                                         quant_scale, quant_offset, convert_to_unsigned, new_file_extension)

    fof_full_path = os.path.join(destination_directory, fof_name)
    with open(fof_full_path, 'w') as fof_file:
        for filename in output_signatures:
            fof_file.write(f'{filename}\n')


if __name__ == "__main__":
    preprocess()



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import os
import subprocess


def escape_special_chars(text, tool=None):
    special_chars = [
        '&', '|', '(', ')'
    ]

    for char in special_chars:
        text = text.replace(char, f'^{char}')

    # handle URL special cases
    if tool != "rclone":
        text = text.replace('%', "%%")

    return text


def preprocess(i):

    os_info = i['os_info']
    env = i['env']

    # env to be passed to the  subprocess
    subprocess_env = os.environ.copy()
    subprocess_env['PATH'] += os.pathsep + \
        os.pathsep.join(env.get('+PATH', ''))

    meta = i['meta']

    automation = i['automation']

    quiet = (env.get('CM_QUIET', False) == 'yes')

    tool = env.get('CM_DOWNLOAD_TOOL', '')
    pre_clean = env.get('CM_PRE_DOWNLOAD_CLEAN', False)

    #    xsep = '^&^&' if windows else '&&'
    xsep = '&&'

    q = '"' if os_info['platform'] == 'windows' else "'"

    x = '*' if os_info['platform'] == 'windows' else ''
    x_c = '-s' if os_info['platform'] == 'darwin_off' else ''

    # command for deleting file in windows and linux is different
    if os_info['platform'] == 'windows':
        del_cmd = "del /f"
    else:
        del_cmd = "rm -f"

    if env.get('CM_DOWNLOAD_LOCAL_FILE_PATH'):
        filepath = env['CM_DOWNLOAD_LOCAL_FILE_PATH']

        if not os.path.exists(filepath):
            return {'return': 1,
                    'error': 'Local file {} doesn\'t exist'.format(filepath)}

        env['CM_DOWNLOAD_CMD'] = ""

        env['CM_DOWNLOAD_FILENAME'] = filepath

        if not quiet:
            print('')
            print('Using local file: {}'.format(filepath))
    else:
        url = env.get('CM_DOWNLOAD_URL', '')

        if url == '':
            return {
                'return': 1, 'error': 'please specify URL using --url={URL} or --env.CM_DOWNLOAD_URL={URL}'}

        print('')
        print('Downloading from {}'.format(url))

        if '&' in url and tool != "cmutil":
            if os_info['platform'] == 'windows':
                url = '"' + url + '"'
            else:
                url = url.replace('&', '\\&')

        extra_download_options = env.get('CM_DOWNLOAD_EXTRA_OPTIONS', '')

        verify_ssl = env.get('CM_VERIFY_SSL', "True")
        if str(verify_ssl).lower() in [
                "no", "false"] or os_info['platform'] == 'windows':
            verify_ssl = False
        else:
            verify_ssl = True

        if env.get('CM_DOWNLOAD_PATH', '') != '':
            download_path = env['CM_DOWNLOAD_PATH']
            if not os.path.exists(download_path):
                os.makedirs(download_path, exist_ok=True)
            os.chdir(download_path)

        if env.get('CM_DOWNLOAD_FILENAME', '') == '':
            urltail = os.path.basename(env['CM_DOWNLOAD_URL'])
            urlhead = os.path.dirname(env['CM_DOWNLOAD_URL'])
            if "." in urltail and "/" in urlhead:
                # Check if ? after filename
                j = urltail.find('?')
                if j > 0:
                    urltail = urltail[:j]
                env['CM_DOWNLOAD_FILENAME'] = urltail
            elif env.get('CM_DOWNLOAD_TOOL', '') == "rclone":
                env['CM_DOWNLOAD_FILENAME'] = urltail
            else:
                env['CM_DOWNLOAD_FILENAME'] = "index.html"

        if tool == "cmutil":
            cmutil_require_download = 0
            if env.get('CM_DOWNLOAD_CHECKSUM_FILE', '') != '':
                if os_info['platform'] == 'windows':
                    checksum_cmd = f"cd {q}{filepath}{q} {xsep}  md5sum -c{x_c} {x}{escape_special_chars(env['CM_DOWNLOAD_CHECKSUM_FILE'])}"
                else:
                    checksum_cmd = f"cd {q}{filepath}{q} {xsep}  md5sum -c{x_c} {x}{q}{env['CM_DOWNLOAD_CHECKSUM_FILE']}{q}"
                checksum_result = subprocess.run(
                    checksum_cmd,
                    cwd=f'{q}{filepath}{q}',
                    capture_output=True,
                    text=True,
                    shell=True,
                    env=subprocess_env)
            elif env.get('CM_DOWNLOAD_CHECKSUM', '') != '':
                if os_info['platform'] == 'windows':
                    checksum_cmd = f"echo {env.get('CM_DOWNLOAD_CHECKSUM')} {x}{escape_special_chars(env['CM_DOWNLOAD_FILENAME'])} | md5sum -c{x_c} -"
                else:
                    checksum_cmd = f"echo {env.get('CM_DOWNLOAD_CHECKSUM')} {x}{q}{env['CM_DOWNLOAD_FILENAME']}{q} | md5sum -c{x_c} -"
                checksum_result = subprocess.run(
                    checksum_cmd,
                    capture_output=True,
                    text=True,
                    shell=True,
                    env=subprocess_env)
            if env.get('CM_DOWNLOAD_CHECKSUM_FILE', '') != '' or env.get(
                    'CM_DOWNLOAD_CHECKSUM', '') != '':
                # print(checksum_result) #for debugging
                if "checksum did not match" in checksum_result.stderr.lower():
                    computed_checksum = subprocess.run(
                        f"md5sum {env['CM_DOWNLOAD_FILENAME']}",
                        capture_output=True,
                        text=True,
                        shell=True).stdout.split(" ")[0]
                    print(
                        f"WARNING: File already present, mismatch between original checksum({env.get('CM_DOWNLOAD_CHECKSUM')}) and computed checksum({computed_checksum}). Deleting the already present file and downloading new.")
                    try:
                        os.remove(env['CM_DOWNLOAD_FILENAME'])
                        print(
                            f"File {env['CM_DOWNLOAD_FILENAME']} deleted successfully.")
                    except PermissionError:
                        return {
                            "return": 1, "error": f"Permission denied to delete file {env['CM_DOWNLOAD_FILENAME']}."}
                    cmutil_require_download = 1
                elif "no such file" in checksum_result.stderr.lower():
                    # print(f"No file {env['CM_DOWNLOAD_FILENAME']}. Downloading through cmutil.")
                    cmutil_require_download = 1
                elif checksum_result.returncode > 0:
                    return {
                        "return": 1, "error": f"Error while checking checksum: {checksum_result.stderr}"}
                else:
                    print(
                        f"File {env['CM_DOWNLOAD_FILENAME']} already present, original checksum and computed checksum matches! Skipping Download..")
            else:
                cmutil_require_download = 1

            if cmutil_require_download == 1:
                cm = automation.cmind
                for i in range(1, 5):
                    r = cm.access({'action': 'download_file',
                                   'automation': 'utils,dc2743f8450541e3',
                                   'url': url,
                                   'verify': verify_ssl})
                    if r['return'] == 0:
                        break
                    oldurl = url
                    url = env.get('CM_DOWNLOAD_URL' + str(i), '')
                    if url == '':
                        break
                    print(f"Download from {oldurl} failed, trying from {url}")

                if r['return'] > 0:
                    return r

                env['CM_DOWNLOAD_CMD'] = ""
                env['CM_DOWNLOAD_FILENAME'] = r['filename']

        elif tool == "wget":
            if env.get('CM_DOWNLOAD_FILENAME', '') != '':
                extra_download_options += f" --tries=3 -O {q}{env['CM_DOWNLOAD_FILENAME']}{q} "
                if not verify_ssl:
                    extra_download_options += "--no-check-certificate "
            env['CM_DOWNLOAD_CMD'] = f"wget -nc {extra_download_options} {url}"
            for i in range(1, 5):
                url = env.get('CM_DOWNLOAD_URL' + str(i), '')
                if url == '':
                    break
                env['CM_DOWNLOAD_CMD'] += f" || (({del_cmd} {env['CM_DOWNLOAD_FILENAME']} || true) && wget -nc {extra_download_options} {url})"
            print(env['CM_DOWNLOAD_CMD'])

        elif tool == "curl":
            if env.get('CM_DOWNLOAD_FILENAME', '') != '':
                extra_download_options += f" --output {q}{env['CM_DOWNLOAD_FILENAME']}{q} "

            env['CM_DOWNLOAD_CMD'] = f"curl {extra_download_options} {url}"
            for i in range(1, 5):
                url = env.get('CM_DOWNLOAD_URL' + str(i), '')
                if url == '':
                    break
                env['CM_DOWNLOAD_CMD'] += f" || (({del_cmd} {env['CM_DOWNLOAD_FILENAME']} || true) && curl {extra_download_options} {url})"

        elif tool == "gdown":
            if not verify_ssl:
                extra_download_options += "--no-check-certificate "
            env['CM_DOWNLOAD_CMD'] = f"gdown {extra_download_options} {url}"
            for i in range(1, 5):
                url = env.get('CM_DOWNLOAD_URL' + str(i), '')
                if url == '':
                    break
                env['CM_DOWNLOAD_CMD'] += f" || (({del_cmd} {env['CM_DOWNLOAD_FILENAME']} || true) && gdown {extra_download_options} {url})"

        elif tool == "rclone":
            # keeping this for backward compatibility. Ideally should be done
            # via get,rclone-config script
            if env.get('CM_RCLONE_CONFIG_CMD', '') != '':
                env['CM_DOWNLOAD_CONFIG_CMD'] = env['CM_RCLONE_CONFIG_CMD']
            rclone_copy_using = env.get('CM_RCLONE_COPY_USING', 'sync')
            if rclone_copy_using == "sync":
                pre_clean = False
            if env["CM_HOST_OS_TYPE"] == "windows":
                # have to modify the variable from url to temp_url if it is
                # going to be used anywhere after this point
                url = url.replace("%", "%%")
                temp_download_file = env['CM_DOWNLOAD_FILENAME'].replace(
                    "%", "%%")
                env['CM_DOWNLOAD_CMD'] = f"rclone {rclone_copy_using} {q}{url}{q} {q}{os.path.join(os.getcwd(), temp_download_file)}{q} -P --error-on-no-transfer"
            else:
                env['CM_DOWNLOAD_CMD'] = f"rclone {rclone_copy_using} {q}{url}{q} {q}{os.path.join(os.getcwd(), env['CM_DOWNLOAD_FILENAME'])}{q} -P --error-on-no-transfer"

        filename = env['CM_DOWNLOAD_FILENAME']
        env['CM_DOWNLOAD_DOWNLOADED_FILENAME'] = filename

        filename = os.path.basename(env['CM_DOWNLOAD_FILENAME'])
        filepath = os.path.join(os.getcwd(), filename)

    env['CM_DOWNLOAD_DOWNLOADED_PATH'] = filepath

    # verify checksum if file already present
    if env.get('CM_DOWNLOAD_CHECKSUM_FILE', '') != '':
        env['CM_DOWNLOAD_CHECKSUM_CMD'] = f"cd {q}{filepath}{q} {xsep}  md5sum -c {x_c} {x}{q}{env['CM_DOWNLOAD_CHECKSUM_FILE']}{q}"
    elif env.get('CM_DOWNLOAD_CHECKSUM', '') != '':
        if os_info['platform'] == 'windows':
            env['CM_DOWNLOAD_CHECKSUM_CMD'] = "echo {} {}{} | md5sum {} -c -".format(
                env.get('CM_DOWNLOAD_CHECKSUM'), x, escape_special_chars(
                    env['CM_DOWNLOAD_FILENAME']), x_c)
        else:
            env['CM_DOWNLOAD_CHECKSUM_CMD'] = "echo {} {}{}{}{} | md5sum {} -c -".format(
                env.get('CM_DOWNLOAD_CHECKSUM'), x, q, env['CM_DOWNLOAD_FILENAME'], q, x_c)
        for i in range(1, 5):
            if env.get('CM_DOWNLOAD_CHECKSUM' + str(i), '') == '':
                break
            if os_info['platform'] == 'windows':
                env['CM_DOWNLOAD_CHECKSUM_CMD'] += " || echo {} {}{} | md5sum {} -c -".format(
                    env.get(
                        'CM_DOWNLOAD_CHECKSUM' +
                        str(i)),
                    x,
                    escape_special_chars(
                        env['CM_DOWNLOAD_FILENAME']),
                    x_c)
            else:
                env['CM_DOWNLOAD_CHECKSUM_CMD'] += " || echo {} {}{}{}{} | md5sum {} -c -".format(
                    env.get(
                        'CM_DOWNLOAD_CHECKSUM' +
                        str(i)),
                    x,
                    q,
                    env['CM_DOWNLOAD_FILENAME'].replace(
                        "%",
                        "%%"),
                    q,
                    x_c)
        # print(env['CM_DOWNLOAD_CHECKSUM_CMD'])
    else:
        env['CM_DOWNLOAD_CHECKSUM_CMD'] = ""

    if not pre_clean:
        env['CM_PRE_DOWNLOAD_CMD'] = ''

    if os_info['platform'] == 'windows' and env.get(
            'CM_DOWNLOAD_CMD', '') != '':
        env['CM_DOWNLOAD_CMD'] = escape_special_chars(
            env['CM_DOWNLOAD_CMD'], tool)
        if pre_clean:
            env['CM_PRE_DOWNLOAD_CLEAN_CMD'] = "del /Q %CM_DOWNLOAD_FILENAME%"
        # Check that if empty CMD, should add ""
        for x in ['CM_DOWNLOAD_CMD', 'CM_DOWNLOAD_CHECKSUM_CMD']:
            env[x + '_USED'] = 'YES' if env.get(x, '') != '' else 'NO'
    else:
        env['CM_PRE_DOWNLOAD_CLEAN_CMD'] = "rm -f {}".format(
            env['CM_DOWNLOAD_FILENAME'])

    return {'return': 0}


def postprocess(i):

    automation = i['automation']

    env = i['env']

    filepath = env['CM_DOWNLOAD_DOWNLOADED_PATH']

    if not os.path.exists(filepath):
        return {
            'return': 1, 'error': 'Downloaded path {} does not exist. Probably CM_DOWNLOAD_FILENAME is not set and CM_DOWNLOAD_URL given is not pointing to a file'.format(filepath)}

    if env.get('CM_DOWNLOAD_RENAME_FILE', '') != '':
        file_dir = os.path.dirname(filepath)
        new_file_name = env['CM_DOWNLOAD_RENAME_FILE']
        new_file_path = os.path.join(file_dir, new_file_name)
        os.rename(filepath, new_file_path)
        filepath = new_file_path

    if env.get('CM_DOWNLOAD_FINAL_ENV_NAME', '') != '':
        env[env['CM_DOWNLOAD_FINAL_ENV_NAME']] = filepath

    env['CM_GET_DEPENDENT_CACHED_PATH'] = filepath

    # Since may change directory, check if need to clean some temporal files
    automation.clean_some_tmp_files({'env': env})

    return {'return': 0}



import cmind as cm
import time

times = []

steps = 10

print('Running search with tags {} times ...'.format(steps))

for step in range(steps):

    start = time.time()
    r = cm.access({'action': 'search',
                   'automation': 'script',
                   'tags': 'detect,os'})
    timer = time.time() - start

    if r['return'] > 0:
        cm.error(r)

    times.append(timer)

step = 0
for t in times:
    step += 1
    print("{}) {:0.3f} sec.".format(step, t))



from cmind import utils
import os


def preprocess(i):

    os_info = i['os_info']

    env = i['env']

    if env.get('CM_ML_MODEL_LOCAL', '') == 'yes':
        ml_model = env.get('CM_ML_MODEL_FILENAME', '')
        if ml_model == '':
            return {'return': 1, 'error': '_local.{model name.pth} is not specified'}

        if not os.path.isabs(ml_model):
            ml_model = os.path.join(
                env.get(
                    'CM_TMP_CURRENT_PATH',
                    ''),
                ml_model)

        if not os.path.isfile(ml_model):
            return {'return': 1,
                    'error': 'ML model {} is not found'.format(ml_model)}

        env['CM_ML_MODEL_FILE_WITH_PATH'] = ml_model

    return {'return': 0}


def postprocess(i):

    env = i['env']

    if env.get('CM_ML_MODEL_FILE_WITH_PATH', '') == '':
        env['CM_ML_MODEL_FILE_WITH_PATH'] = 'model-weights-skipped'

    env['CM_ML_MODEL_FILE'] = os.path.basename(
        env['CM_ML_MODEL_FILE_WITH_PATH'])

    if env.get('CM_ABTF_SSD_PYTORCH', '') == '':
        env['CM_ABTF_SSD_PYTORCH'] = 'model-code-skipped'

    env['CM_ML_MODEL_CODE_WITH_PATH'] = env['CM_ABTF_SSD_PYTORCH']

    env['CM_GET_DEPENDENT_CACHED_PATH'] = env['CM_ML_MODEL_FILE_WITH_PATH']

    return {'return': 0}



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import os


def preprocess(i):

    os_info = i['os_info']

    env = i['env']

    meta = i['meta']

    automation = i['automation']

    quiet = (env.get('CM_QUIET', False) == 'yes')

    ############################################################
    cur_dir = os.getcwd()

    name = env.get('CM_NAME', '')
    if name == '':
        artifacts = i.get('input', {}).get('artifacts', [])
        if len(artifacts) > 0:
            name = artifacts[0]
    if name == '':
        name = 'default'

    if os_info['platform'] == 'windows':
        activate_script = os.path.join('Scripts', 'activate.bat')
    else:
        activate_script = os.path.join('bin', 'activate')

    activate_script2 = os.path.join(name, activate_script)

    if not os.path.isfile(activate_script2):
        force_python_path = env.get('CM_SET_VENV_PYTHON', '')

        if force_python_path != '' and not os.path.isfile(force_python_path):
            return {'return': 1, 'error': 'python executable not found: {}'.format(
                force_python_path)}

        if os_info['platform'] == 'windows':
            python_path = 'python.exe' if force_python_path == '' else force_python_path
            create_dir = ' & md {}\\work'
        else:
            python_path = 'python3' if force_python_path == '' else force_python_path
            create_dir = ' ; mkdir {}/work'

        cmd = python_path + ' -m venv ' + name + create_dir.format(name)

        print('====================================================================')

        print('Creating venv: "{}" ...'.format(cmd))
        os.system(cmd)

    if os.path.isfile(activate_script2):
        script_file = 'venv-' + name
        if os_info['platform'] == 'windows':
            script_file += '.bat'
            xcmd = script_file
        else:
            script_file += '.sh'
            xcmd = 'source ' + script_file

        if not os.path.isfile(script_file):

            work_dir = os.path.join(name, 'work')
            if not os.path.isdir(work_dir):
                os.makedirs(work_dir)

            if os_info['platform'] == 'windows':
                shell = os.environ.get('CM_SET_VENV_SHELL', '')
                if shell == '':
                    shell = env.get('CM_SET_VENV_SHELL', '')
                if shell != '':
                    shell = shell.replace('CM_SET_VENV_WORK', 'work')
                if shell == '':
                    shell = 'cmd'
                cmd = 'cd {} & call {} & set CM_REPOS=%CD%\\{}\\CM & {}\n'.format(
                    name, activate_script, name, shell)
            else:
                cmd = '#!/bin/bash\n\ncd {} ; source {} ; export CM_REPOS=$PWD/CM ; cd work\n'.format(
                    name, activate_script)

            with open(script_file, 'w') as f:
                f.write(cmd)

        print('====================================================================')
        print('Please run the following command:')
        print('')
        print(xcmd)
        print('====================================================================')

    return {'return': 0}


def postprocess(i):

    env = i['env']

    return {'return': 0}



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import os
import shutil


def preprocess(i):

    os_info = i['os_info']

    if os_info['platform'] == 'windows':
        return {'return': 1, 'error': 'Windows is not supported in this script yet'}
    env = i['env']

    conf = env.get('CM_MLPERF_NVIDIA_TRAINING_SYSTEM_CONF_NAME', '')
    if conf == "":
        return {'return': 1,
                'error': 'Please provide --system_conf_name=<CONF_SOURCE_FILE>'}

    if not conf.endswith(".sh"):
        conf = conf + ".sh"

    if env.get('CM_MLPERF_TRAINING_BENCHMARK', '') == "resnet":
        i['run_script_input']['script_name'] = "run-resnet"

    env['CONFIG_FILE'] = conf
#    print(env)

    return {'return': 0}


def postprocess(i):

    env = i['env']

    return {'return': 0}



import abc
import concurrent.futures
import logging
import multiprocessing
import threading
import typing

from loadgen.harness import ModelRunner, QueryInput, QueryResult
from loadgen.model import Model, ModelFactory, ModelInput

logger = logging.getLogger(__name__)

# Runner implementations


class ModelRunnerInline(ModelRunner):
    def __init__(self, model_factory: ModelFactory):
        self.model = model_factory.create()

    def issue_query(self, queries: QueryInput) -> typing.Optional[QueryResult]:
        result = dict()
        for query_id, model_input in queries.items():
            output = self.model.predict(model_input)
            result[query_id] = output
        return result


class ModelRunnerPoolExecutor(ModelRunner):
    def __init__(self):
        self.executor: concurrent.futures.Executor = None
        self.futures = None

    def __exit__(self, _exc_type, _exc_value, _traceback):
        if self.executor:
            self.executor.shutdown(True)
        return super().__exit__(_exc_type, _exc_value, _traceback)

    def issue_query(self, queries: QueryInput) -> typing.Optional[QueryResult]:
        self.futures = dict()
        predictor_fn = self.get_predictor()
        for query_id, model_input in queries.items():
            f = self.executor.submit(predictor_fn, model_input)
            self.futures[f] = query_id
        return None

    def flush_queries(self) -> typing.Optional[QueryResult]:
        result = dict()
        for future in concurrent.futures.as_completed(self.futures.keys()):
            query_id = self.futures[future]
            query_result = future.result()
            result[query_id] = query_result
        return result

    @abc.abstractmethod
    def get_predictor(self) -> typing.Callable[[ModelInput], typing.Any]:
        pass


class ModelRunnerThreadPoolExecutor(ModelRunnerPoolExecutor):
    def __init__(self, model_factory: ModelFactory, max_concurrency: int):
        super().__init__()
        self.model = model_factory.create()
        self.max_concurrency = max_concurrency

    def __enter__(self):
        self.executor = concurrent.futures.ThreadPoolExecutor(
            max_workers=self.max_concurrency, thread_name_prefix="LoadGen"
        )
        return self

    def get_predictor(self) -> typing.Callable[[ModelInput], typing.Any]:
        return self.model.predict


class ModelRunnerThreadPoolExecutorWithTLS(ModelRunnerPoolExecutor):
    tls: threading.local

    def __init__(self, model_factory: ModelFactory, max_concurrency: int):
        super().__init__()
        self.model_factory = model_factory
        self.max_concurrency = max_concurrency

    def __enter__(self):
        self.executor = concurrent.futures.ThreadPoolExecutor(
            max_workers=self.max_concurrency,
            thread_name_prefix="LoadGen",
            initializer=ModelRunnerThreadPoolExecutorWithTLS._tls_init,
            initargs=(self.model_factory,),
        )
        return self

    def get_predictor(self) -> typing.Callable[[ModelInput], typing.Any]:
        return ModelRunnerThreadPoolExecutorWithTLS._tls_predict

    @staticmethod
    def _tls_init(model_factory: ModelFactory):
        ModelRunnerThreadPoolExecutorWithTLS.tls = threading.local()
        ModelRunnerThreadPoolExecutorWithTLS.tls.model = model_factory.create()

    @staticmethod
    def _tls_predict(input: ModelInput):
        return ModelRunnerThreadPoolExecutorWithTLS.tls.model.predict(input)


class ModelRunnerProcessPoolExecutor(ModelRunnerPoolExecutor):
    _model: Model

    def __init__(self, model_factory: ModelFactory, max_concurrency: int):
        super().__init__()
        self.max_concurrency = max_concurrency
        ModelRunnerProcessPoolExecutor._model = model_factory.create()

    def __enter__(self):
        self.executor = concurrent.futures.ProcessPoolExecutor(
            max_workers=self.max_concurrency
        )
        return self

    def get_predictor(self) -> typing.Callable[[ModelInput], typing.Any]:
        return ModelRunnerProcessPoolExecutor._predict

    @staticmethod
    def _predict(input: ModelInput):
        result = ModelRunnerProcessPoolExecutor._model.predict(input)
        return result


class ModelRunnerMultiProcessingPool(ModelRunner):
    _model: Model

    def __init__(
        self,
        model_factory: ModelFactory,
        max_concurrency: int,
    ):
        self.max_concurrency = max_concurrency
        self.task: multiprocessing.ApplyResult = None
        ModelRunnerMultiProcessingPool._model = model_factory.create()

    def __enter__(self):
        self.pool = multiprocessing.Pool(self.max_concurrency)

    def __exit__(self, _exc_type, _exc_value, _traceback):
        if self.pool:
            self.pool.terminate()
        return super().__exit__(_exc_type, _exc_value, _traceback)

    def issue_query(self, queries: QueryInput) -> typing.Optional[QueryResult]:
        if hasattr(self, "tasks"):
            assert len(self.tasks) == 0
            for query_id, model_input in queries.items():
                task = self.pool.apply_async(
                    ModelRunnerMultiProcessingPool._predict, (model_input,)
                )
                self.tasks[task] = query_id
        else:
            assert self.task is None
            inputs = [
                [query_id, model_input] for query_id, model_input in queries.items()
            ]
            self.task = self.pool.starmap_async(
                ModelRunnerMultiProcessingPool._predict_with_id, inputs
            )
            return None

    def flush_queries(self) -> typing.Optional[QueryResult]:
        if hasattr(self, "tasks"):
            result = dict()
            for task, query_id in self.tasks.items():
                task_result = task.get()
                result[query_id] = task_result
            return result
        else:
            task_result = self.task.get()
            result = {
                query_id: query_result for query_id,
                query_result in task_result}
            return result

    @staticmethod
    def _predict(input: ModelInput):
        result = ModelRunnerMultiProcessingPool._model.predict(input)
        return result

    @staticmethod
    def _predict_with_id(query_id: int, input: ModelInput):
        result = ModelRunnerMultiProcessingPool._model.predict(input)
        return (query_id, result)



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import os
import subprocess


def check_installation(command, os_info):
    if os_info['platform'] == "windows":
        return subprocess.call(
            [command, '--version'], stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True) == 0
    elif os_info['platform'] == "linux":
        return subprocess.call(['which', command], stdout=subprocess.PIPE,
                               stderr=subprocess.PIPE) == 0  # 0 means the package is there


def preprocess(i):

    os_info = i['os_info']
    env = i['env']

    if not check_installation("numactl", os_info):
        env['CM_INSTALL_NUMACTL'] = 'True'

    # if not check_installation("cpupower",os_info):
    env['CM_INSTALL_CPUPOWER'] = 'True'

    if env.get('CM_PLATFORM_DETAILS_FILE_PATH', '') == '':
        if env.get('CM_PLATFORM_DETAILS_DIR_PATH', '') == '':
            env['CM_PLATFORM_DETAILS_DIR_PATH'] = os.getcwd()
        if env.get('CM_PLATFORM_DETAILS_FILE_NAME', '') == '':
            env['CM_PLATFORM_DETAILS_FILE_NAME'] = "system-info.txt"
        env['CM_PLATFORM_DETAILS_FILE_PATH'] = os.path.join(
            env['CM_PLATFORM_DETAILS_DIR_PATH'], env['CM_PLATFORM_DETAILS_FILE_NAME'])

    return {'return': 0}


def postprocess(i):

    state = i['state']

    env = i['env']

    os_info = i['os_info']

    automation = i['automation']

    return {'return': 0}



import sys
import os
import cmind as cm
import check as checks
import json
import yaml

files = sys.argv[1:]

for file in files:
    if not os.path.isfile(file):
        continue
    if not file.endswith("_cm.json") and not file.endswith("_cm.yaml"):
        continue
    if not file.startswith(os.path.join("script")):
        continue
    script_path = os.path.dirname(file)
    f = open(file)
    if file.endswith(".json"):
        data = json.load(f)
    elif file.endswith(".yaml"):
        data = yaml.safe_load(f)
    uid = data['uid']

    r = cm.access({'action': 'doc', 'automation': 'script',
                  'artifact': uid, 'quiet': 'yes'})
    checks.check_return(r)



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import os
import json
import shutil


def preprocess(i):
    return generate_submission(i)


##############################################################################

def generate_submission(i):

    # Save current user directory
    cur_dir = os.getcwd()
    env = i['env']
    state = i['state']
    inp = i['input']
    results_dir = env['CM_MLPERF_RESULTS_DIR']

    if 'CM_MLPERF_SUBMISSION_DIR' not in env:
        env['CM_MLPERF_SUBMISSION_DIR'] = os.path.join(cur_dir, "results")
    submission_dir = env['CM_MLPERF_SUBMISSION_DIR']
    if not os.path.isdir(submission_dir):
        os.makedirs(submission_dir)

    print('* MLPerf tiny submission dir: {}'.format(submission_dir))
    print('* MLPerf tiny results dir: {}'.format(results_dir))
    results = [
        f for f in os.listdir(results_dir) if not os.path.isfile(
            os.path.join(
                results_dir,
                f))]

    division = inp.get('division', 'open')

    if division not in ['open', 'closed']:
        return {'return': 1, 'error': '"division" must be "open" or "closed"'}
    system_meta = state['CM_SUT_META']
    division = system_meta['division']

    print('* MLPerf tiny division: {}'.format(division))

    path_submission_root = submission_dir
    path_submission_division = os.path.join(path_submission_root, division)
    if not os.path.isdir(path_submission_division):
        os.makedirs(path_submission_division)

    # Check submitter
    submitter = system_meta['submitter']
    env['CM_MLPERF_SUBMITTER'] = submitter

    print('* MLPerf tiny submitter: {}'.format(submitter))

    path_submission = os.path.join(path_submission_division, submitter)
    if not os.path.isdir(path_submission):
        os.makedirs(path_submission)

    # SUT base
    system = i.get('system', 'default')

    code_path = os.path.join(path_submission, "code")
    for res in results:
        parts = res.split("-")
        backend = parts[0]
        target = parts[1]
        framework = backend

        print('* Target: {}'.format(target))
        print('* Framework: {}'.format(framework))
        result_path = os.path.join(results_dir, res)
        platform_prefix = inp.get('platform_prefix', '')
        if platform_prefix:
            sub_res = platform_prefix + "-" + res
        else:
            sub_res = res
        submission_path = os.path.join(path_submission, "results", sub_res)
        measurement_path = os.path.join(
            path_submission, "measurements", sub_res)
        compliance_path = os.path.join(path_submission, "compliance", sub_res)
        system_path = os.path.join(path_submission, "systems")
        submission_system_path = system_path
        if not os.path.isdir(submission_system_path):
            os.makedirs(submission_system_path)
        system_file = os.path.join(submission_system_path, sub_res + ".json")
        with open(system_file, "w") as fp:
            json.dump(system_meta, fp, indent=2)

        models = [
            f for f in os.listdir(result_path) if not os.path.isfile(
                os.path.join(
                    result_path, f))]
        for model in models:
            result_model_path = os.path.join(result_path, model)
            submission_model_path = os.path.join(submission_path, model)
            measurement_model_path = os.path.join(measurement_path, model)
            compliance_model_path = os.path.join(compliance_path, model)
            code_model_path = os.path.join(code_path, model)
            scenarios = [
                f for f in os.listdir(result_model_path) if not os.path.isfile(
                    os.path.join(
                        result_model_path, f))]
            submission_code_path = code_model_path
            if not os.path.isdir(submission_code_path):
                os.makedirs(submission_code_path)
            if not os.path.exists(os.path.join(
                    submission_code_path, "README.md")):
                with open(os.path.join(submission_code_path, "README.md"), mode='w'):
                    pass  # create an empty README

            print('* MLPerf inference model: {}'.format(model))
            for scenario in scenarios:
                result_scenario_path = os.path.join(
                    result_model_path, scenario)
                submission_scenario_path = os.path.join(
                    submission_model_path, scenario)
                measurement_scenario_path = os.path.join(
                    measurement_model_path, scenario)
                compliance_scenario_path = os.path.join(
                    compliance_model_path, scenario)

                modes = [
                    f for f in os.listdir(result_scenario_path) if not os.path.isfile(
                        os.path.join(
                            result_scenario_path, f))]
                for mode in modes:
                    result_mode_path = os.path.join(result_scenario_path, mode)
                    submission_mode_path = os.path.join(
                        submission_scenario_path, mode)
                    submission_results_path = submission_mode_path
                    submission_measurement_path = measurement_scenario_path
                    submission_compliance_path = os.path.join(
                        compliance_scenario_path, mode)
                    if mode == 'performance':
                        result_mode_path = os.path.join(
                            result_mode_path, 'run_1')
                        submission_results_path = os.path.join(
                            submission_mode_path, 'run_1')
                    if not os.path.isdir(submission_results_path):
                        os.makedirs(submission_results_path)
                    if not os.path.isdir(submission_measurement_path):
                        os.makedirs(submission_measurement_path)
                    if not os.path.isdir(submission_compliance_path):
                        os.makedirs(submission_compliance_path)
                    mlperf_inference_conf_path = os.path.join(
                        result_mode_path, "mlperf.conf")
                    if os.path.exists(mlperf_inference_conf_path):
                        shutil.copy(
                            mlperf_inference_conf_path, os.path.join(
                                submission_measurement_path, 'mlperf.conf'))
                    user_conf_path = os.path.join(
                        result_mode_path, "user.conf")
                    if os.path.exists(user_conf_path):
                        shutil.copy(
                            user_conf_path, os.path.join(
                                submission_measurement_path, 'user.conf'))
                    measurements_json_path = os.path.join(
                        result_mode_path, "measurements.json")
                    if os.path.exists(user_conf_path):
                        shutil.copy(
                            measurements_json_path,
                            os.path.join(
                                submission_measurement_path,
                                sub_res + '.json'))
                    files = []
                    readme = False
                    for f in os.listdir(result_mode_path):
                        if f.startswith('mlperf_'):
                            files.append(f)
                        if f == "README.md":
                            shutil.copy(
                                os.path.join(
                                    result_mode_path, f), os.path.join(
                                    submission_measurement_path, f))
                            readme = True

                    if mode == "accuracy":
                        if os.path.exists(os.path.join(
                                result_mode_path, "accuracy.txt")):
                            files.append("accuracy.txt")

                    for f in files:
                        print(' * ' + f)
                        p_target = os.path.join(submission_results_path, f)
                        shutil.copy(
                            os.path.join(
                                result_mode_path,
                                f),
                            p_target)

                    if not readme:
                        with open(os.path.join(submission_measurement_path, "README.md"), mode='w'):
                            pass  # create an empty README

    return {'return': 0}


def postprocess(i):

    env = i['env']

    return {'return': 0}



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import os


def preprocess(i):
    os_info = i['os_info']

    env = i['env']
    script_path = i['run_script_input']['path']

    env["CM_SOURCE_FOLDER_PATH"] = script_path
    env['CM_C_SOURCE_FILES'] = "susan.c"

    if 'CM_INPUT' not in env:
        env['CM_INPUT'] = os.path.join(script_path, 'data.pgm')

    if 'CM_OUTPUT' not in env:
        env['CM_OUTPUT'] = 'output_image_with_corners.pgm'

    if 'CM_RUN_DIR' not in env:
        output_path = os.path.join(script_path, "output")
        if output_path != '' and not os.path.isdir(output_path):
            os.makedirs(output_path)

        env['CM_RUN_DIR'] = output_path

    env['CM_RUN_SUFFIX'] = env['CM_INPUT'] + ' ' + env['CM_OUTPUT'] + ' -c'

    if os_info['platform'] == 'windows':
        env['CM_BIN_NAME'] = 'image-corner.exe'
    else:
        env['CM_BIN_NAME'] = 'image-corner'
        env['+ LDCFLAGS'] = ["-lm"]

    return {'return': 0}


def postprocess(i):

    env = i['env']
    print(env['CM_OUTPUT'] + " generated in " + env['CM_RUN_DIR'])

    return {'return': 0}



from cmind import utils
import os
import json
import shutil
import subprocess
import cmind as cm
import copy
from tabulate import tabulate

summary_ext = ['.csv', '.json', '.xlsx']

##########################################################################


def preprocess(i):

    os_info = i['os_info']
    env = i['env']

    inp = i['input']
    state = i['state']
    script_path = i['run_script_input']['path']

    if env.get('CM_RUN_DOCKER_CONTAINER', '') == "yes":
        return {'return': 0}

    dump_version_info = env.get('CM_DUMP_VERSION_INFO', True)
    system_meta = state.get('CM_SUT_META', {})
    if system_meta:
        env['CM_SUT_META_EXISTS'] = "yes"

    env['CM_MODEL'] = env['CM_MLPERF_MODEL']

    # Clean MLPerf inference output tar file if non-standard
    x = env.get('MLPERF_INFERENCE_SUBMISSION_TAR_FILE', '')
    if x != '' and os.path.isfile(x):
        os.remove(x)

    # Clean MLPerf inference submission summary files
    x = env.get('MLPERF_INFERENCE_SUBMISSION_SUMMARY', '')
    if x != '':
        for y in summary_ext:
            z = x + y
            if os.path.isfile(z):
                os.remove(z)

    if env.get('CM_MLPERF_SUBMISSION_SYSTEM_TYPE', '') != '':
        system_type = env['CM_MLPERF_SUBMISSION_SYSTEM_TYPE']
        system_meta['system_type'] = system_type

    if env.get('CM_MLPERF_SUBMISSION_DIVISION', '') != '':
        division = env['CM_MLPERF_SUBMISSION_DIVISION']
        system_meta['division'] = division

    if system_meta.get('division', '') != "closed":
        # no compliance runs needed for open division
        env["CM_MLPERF_LOADGEN_COMPLIANCE"] = "no"

    clean = False

    if 'CM_MLPERF_CLEAN_ALL' in env:
        clean = True
        if 'CM_MLPERF_CLEAN_SUBMISSION_DIR' not in env:
            env['CM_MLPERF_CLEAN_SUBMISSION_DIR'] = "yes"
        if 'CM_RERUN' not in env:
            env['CM_RERUN'] = "yes"

    if str(env.get('CM_SYSTEM_POWER', 'no')).lower(
    ) != "no" or env.get('CM_MLPERF_POWER', '') == "yes":
        power_variation = ",_power"
        env['CM_MLPERF_POWER'] = "yes"
    else:
        power_variation = ""

    if env.get('CM_RUN_STYLE',
               '') == "valid" and 'CM_RUN_MLPERF_ACCURACY' not in env:
        env['CM_RUN_MLPERF_ACCURACY'] = "on"

    if env.get('CM_MLPERF_INFERENCE_SOURCE', '') != '':
        print(
            "Using MLCommons Inference source from " +
            env['CM_MLPERF_INFERENCE_SOURCE'])

    if 'CM_MLPERF_LOADGEN_EXTRA_OPTIONS' not in env:
        env['CM_MLPERF_LOADGEN_EXTRA_OPTIONS'] = ""

    if 'CM_MLPERF_LOADGEN_MODES' not in env:
        if 'CM_MLPERF_LOADGEN_MODE' not in env:
            env['CM_MLPERF_LOADGEN_MODE'] = "performance"

    if 'CM_MLPERF_LOADGEN_SCENARIOS' not in env:
        if 'CM_MLPERF_LOADGEN_SCENARIO' not in env:
            env['CM_MLPERF_LOADGEN_SCENARIO'] = "Offline"

    if env.get('CM_MLPERF_LOADGEN_ALL_SCENARIOS', '') == "yes":
        env['CM_MLPERF_LOADGEN_SCENARIOS'] = get_valid_scenarios(
            env['CM_MODEL'],
            system_meta['system_type'],
            env['CM_MLPERF_LAST_RELEASE'],
            env['CM_MLPERF_INFERENCE_SOURCE'])
    else:
        system_meta = {}
        env['CM_MLPERF_LOADGEN_SCENARIOS'] = [
            env['CM_MLPERF_LOADGEN_SCENARIO']]

    if env.get('CM_MLPERF_LOADGEN_ALL_MODES', '') == "yes":
        env['CM_MLPERF_LOADGEN_MODES'] = ["performance", "accuracy"]
    else:
        env['CM_MLPERF_LOADGEN_MODES'] = [env['CM_MLPERF_LOADGEN_MODE']]

    if env.get('OUTPUT_BASE_DIR', '') == '':
        env['OUTPUT_BASE_DIR'] = env.get(
            'CM_MLPERF_INFERENCE_RESULTS_DIR', os.getcwd())

    test_list = []

    variation_implementation = "_" + \
        env.get("CM_MLPERF_IMPLEMENTATION", "reference")
    variation_model = ",_" + env["CM_MLPERF_MODEL"]
    variation_backend = ",_" + \
        env["CM_MLPERF_BACKEND"] if env.get(
            "CM_MLPERF_BACKEND", "") != "" else ""
    variation_device = ",_" + \
        env["CM_MLPERF_DEVICE"] if env.get(
            "CM_MLPERF_DEVICE", "") != "" else ""
    variation_run_style = ",_" + env.get("CM_MLPERF_RUN_STYLE", "test")
    variation_reproducibility = ",_" + env["CM_RUN_MLPERF_INFERENCE_APP_DEFAULTS"] if env.get(
        "CM_RUN_MLPERF_INFERENCE_APP_DEFAULTS", "") != "" else ""

    if env.get("CM_MLPERF_MODEL_PRECISION", '') != '':
        variation_quantization_string = ",_" + env["CM_MLPERF_MODEL_PRECISION"]
    else:
        variation_quantization_string = ""

    tags = "app,abtf-inference," + variation_implementation + variation_model + variation_backend + variation_device + \
        variation_run_style + variation_reproducibility + \
        variation_quantization_string + power_variation
    verbose = inp.get('v', False)
    print_env = inp.get('print_env', False)
    print_deps = inp.get('print_deps', False)
    add_deps_recursive = inp.get('add_deps_recursive', {})
    add_deps = inp.get('add_deps', {})
    ad = inp.get('ad', {})
    adr = inp.get('adr', {})
    adr_from_meta = i['run_script_input'].get('add_deps_recursive')

    for key in adr_from_meta:
        add_deps_recursive[key] = adr_from_meta[key]

    if env.get('CM_MLPERF_LOADGEN_MAX_BATCHSIZE', '') != '':
        if not add_deps_recursive.get('mlperf-inference-implementation', {}):
            add_deps_recursive['mlperf-inference-implementation'] = {}
        if add_deps_recursive['mlperf-inference-implementation'].get(
                'tags', '') == '':
            add_deps_recursive['mlperf-inference-implementation']['tags'] = ''
        else:
            add_deps_recursive['mlperf-inference-implementation']['tags'] += ','
        add_deps_recursive['mlperf-inference-implementation']['tags'] += "_batch_size." + \
            env['CM_MLPERF_LOADGEN_MAX_BATCHSIZE']

    if env.get('CM_MLPERF_INFERENCE_SUT_VARIATION', '') != '':
        if not add_deps_recursive.get('mlperf-inference-implementation', {}):
            add_deps_recursive['mlperf-inference-implementation'] = {}
        if add_deps_recursive['mlperf-inference-implementation'].get(
                'tags', '') == '':
            add_deps_recursive['mlperf-inference-implementation']['tags'] = ''
        else:
            add_deps_recursive['mlperf-inference-implementation']['tags'] += ','
        add_deps_recursive['mlperf-inference-implementation']['tags'] += "_" + \
            env['CM_MLPERF_INFERENCE_SUT_VARIATION']

    if env.get('CM_NETWORK_LOADGEN', '') != '':
        if not add_deps_recursive.get('mlperf-inference-implementation', {}):
            add_deps_recursive['mlperf-inference-implementation'] = {}
        network_variation_tag = f"_network-{env['CM_NETWORK_LOADGEN']}"
        if add_deps_recursive['mlperf-inference-implementation'].get(
                'tags', '') == '':
            add_deps_recursive['mlperf-inference-implementation']['tags'] = ''
        else:
            add_deps_recursive['mlperf-inference-implementation']['tags'] += ','
        add_deps_recursive['mlperf-inference-implementation']['tags'] += network_variation_tag

    if env.get('CM_OUTPUT_FOLDER_NAME', '') == '':
        env['CM_OUTPUT_FOLDER_NAME'] = env['CM_MLPERF_RUN_STYLE'] + "_results"

    output_dir = os.path.join(
        env['OUTPUT_BASE_DIR'],
        env['CM_OUTPUT_FOLDER_NAME'])
    if clean:
        path_to_clean = output_dir

        print('=========================================================')
        print('Cleaning results in {}'.format(path_to_clean))
        if os.path.exists(path_to_clean):
            shutil.rmtree(path_to_clean)

        print('=========================================================')

    if str(env.get('CM_MLPERF_USE_DOCKER', '')
           ).lower() in ["1", "true", "yes"]:
        action = "docker"
        del (env['OUTPUT_BASE_DIR'])
        state = {}
        docker_extra_input = {}

        if env.get('CM_HW_NAME'):
            del (env['CM_HW_NAME'])

        for k in inp:
            if k.startswith("docker_"):
                docker_extra_input[k] = inp[k]
        inp = {}
    else:
        action = "run"

    # local_keys = [ 'CM_MLPERF_SKIP_RUN', 'CM_MLPERF_LOADGEN_QUERY_COUNT',
    # 'CM_MLPERF_LOADGEN_TARGET_QPS', 'CM_MLPERF_LOADGEN_TARGET_LATENCY' ]

    for scenario in env['CM_MLPERF_LOADGEN_SCENARIOS']:
        scenario_tags = tags + ",_" + scenario.lower()
        env['CM_MLPERF_LOADGEN_SCENARIO'] = scenario

        if scenario == "Offline":
            if env.get('CM_MLPERF_LOADGEN_OFFLINE_TARGET_QPS'):
                env['CM_MLPERF_LOADGEN_TARGET_QPS'] = env['CM_MLPERF_LOADGEN_OFFLINE_TARGET_QPS']
        elif scenario == "Server":
            if env.get('CM_MLPERF_LOADGEN_SERVER_TARGET_QPS'):
                env['CM_MLPERF_LOADGEN_TARGET_QPS'] = env['CM_MLPERF_LOADGEN_SERVER_TARGET_QPS']
        elif scenario == "SingleStream":
            if env.get('CM_MLPERF_LOADGEN_SINGLESTREAM_TARGET_LATENCY'):
                env['CM_MLPERF_LOADGEN_TARGET_LATENCY'] = env['CM_MLPERF_LOADGEN_SINGLESTREAM_TARGET_LATENCY']
        elif scenario == "MultiStream":
            if env.get('CM_MLPERF_LOADGEN_MULTISTREAM_TARGET_LATENCY'):
                env['CM_MLPERF_LOADGEN_TARGET_LATENCY'] = env['CM_MLPERF_LOADGEN_MULTISTREAM_TARGET_LATENCY']

        for mode in env['CM_MLPERF_LOADGEN_MODES']:
            env_copy = copy.deepcopy(env)
            env_copy['CM_MLPERF_LOADGEN_MODE'] = mode
            for key in env_copy:
                if isinstance(env_copy[key], str) and env_copy[key].startswith(
                        "CM_TMP_"):
                    del env_copy[key]

            print(f"\nRunning loadgen scenario: {scenario} and mode: {mode}")
            ii = {'action': action, 'automation': 'script', 'tags': scenario_tags, 'quiet': 'true',
                  'env': env_copy, 'input': inp, 'state': state, 'add_deps': copy.deepcopy(add_deps), 'add_deps_recursive':
                  copy.deepcopy(add_deps_recursive), 'ad': ad, 'adr': copy.deepcopy(adr), 'v': verbose, 'print_env': print_env, 'print_deps': print_deps, 'dump_version_info': dump_version_info}

            if action == "docker":
                for k in docker_extra_input:
                    ii[k] = docker_extra_input[k]
            r = cm.access(ii)
            if r['return'] > 0:
                return r
            if action == "docker":
                # We run commands interactively inside the docker container
                return {'return': 0}

            if env_copy.get('CM_OUTPUT_PREDICTIONS_PATH'):
                print(
                    f"\nOutput predictions can be seen by opening the images inside {env_copy['CM_OUTPUT_PREDICTIONS_PATH']}\n")

            if state.get('docker', {}):
                del (state['docker'])

        if env.get("CM_MLPERF_LOADGEN_COMPLIANCE", "") == "yes":
            for test in test_list:
                env_copy = copy.deepcopy(env)
                for key in env_copy:
                    if isinstance(env_copy[key], str) and env_copy[key].startswith(
                            "CM_TMP_"):
                        del env_copy[key]
                env_copy['CM_MLPERF_LOADGEN_COMPLIANCE_TEST'] = test
                env_copy['CM_MLPERF_LOADGEN_MODE'] = "compliance"
                ii = {'action': action, 'automation': 'script', 'tags': scenario_tags, 'quiet': 'true',
                      'env': env_copy, 'input': inp, 'state': state, 'add_deps': copy.deepcopy(add_deps), 'add_deps_recursive':
                      copy.deepcopy(add_deps_recursive), 'adr': copy.deepcopy(adr), 'ad': ad, 'v': verbose, 'print_env': print_env, 'print_deps': print_deps, 'dump_version_info': dump_version_info}
                if action == "docker":
                    for k in docker_extra_input:
                        ii[k] = docker_extra_input[k]
                r = cm.access(ii)
                if r['return'] > 0:
                    return r
                if state.get('docker', {}):
                    del (state['docker'])

    if state.get("cm-mlperf-inference-results"):
        # print(state["cm-mlperf-inference-results"])
        for sut in state["cm-mlperf-inference-results"]:  # only one sut will be there
            # Better to do this in a stand alone CM script with proper deps but
            # currently we manage this by modifying the sys path of the python
            # executing CM
            import mlperf_utils  # noqa

            print(sut)
            result_table, headers = mlperf_utils.get_result_table(
                state["cm-mlperf-inference-results"][sut])
            print(tabulate(result_table, headers=headers, tablefmt="pretty"))

            print(
                f"\nThe MLPerf inference results are stored at {output_dir}\n")

    return {'return': 0}


def get_valid_scenarios(model, category, mlperf_version, mlperf_path):

    import sys

    submission_checker_dir = os.path.join(mlperf_path, "tools", "submission")

    sys.path.append(submission_checker_dir)
    if not os.path.exists(os.path.join(
            submission_checker_dir, "submission_checker.py")):
        shutil.copy(os.path.join(submission_checker_dir, "submission-checker.py"), os.path.join(submission_checker_dir,
                                                                                                "submission_checker.py"))

    import submission_checker as checker

    if "dlrm-99" in model:
        model = model.replace("dlrm-99", "dlrm-v2-99")
    if "sdxl" in model:
        model = "stable-diffusion-xl"

    config = checker.MODEL_CONFIG

    internal_model_name = config[mlperf_version]["model_mapping"].get(
        model, model)

    valid_scenarios = config[mlperf_version]["required-scenarios-" +
                                             category][internal_model_name]

    print(
        "Valid Scenarios for " +
        model +
        " in " +
        category +
        " category are :" +
        str(valid_scenarios))

    return valid_scenarios

##########################################################################


def postprocess(i):

    env = i['env']
    state = i['state']

    if env.get('CM_MLPERF_IMPLEMENTATION', '') == 'reference':
        x1 = env.get('CM_MLPERF_INFERENCE_SOURCE', '')
        x2 = env.get('CM_MLPERF_INFERENCE_CONF_PATH', '')

        if x1 != '' and x2 != '':
            print('')
            print(
                'Path to the MLPerf inference benchmark reference sources: {}'.format(x1))
            print(
                'Path to the MLPerf inference reference configuration file: {}'.format(x2))
            print('')

    return {'return': 0}


##########################################################################


def load_md(path, path2, name):

    fn = os.path.join(path, path2, name + '.md')

    s = ''

    if os.path.isfile(fn):
        r = utils.load_txt(fn)
        if r['return'] > 0:
            return r

        s = r['string']

    return {'return': 0, 'string': s}

##########################################################################


def get_url(url, path, path2, name, text):

    name_md = name + '.md'
    fn = os.path.join(path, path2, name_md)

    urlx = ''
    url_online = ''
    if os.path.isfile(fn):
        if not url.endswith('/'):
            url += '/'
        urlx = url + path2 + '/' + name_md

        url_online = '[{}]({})'.format(text, urlx)

    return {'return': 0, 'url_online': url_online}

##########################################################################



# Developer(s): Grigori Fursin

import cmind
import os
import datetime
import misc


def page(st, params):

    url_prefix = st.config.get_option('server.baseUrlPath') + '/'
    url_prefix_script = url_prefix + '?action=scripts'

    script_name = ''
    x = params.get('name', [''])
    if len(x) > 0 and x[0] != '':
        script_name = x[0].strip()

    script_tags = ''
    if script_name == '':
        x = params.get('tags', [''])
        if len(x) > 0 and x[0] != '':
            script_tags = x[0].strip()

    if script_tags == 'modular,app':
        x = '''
             <i>
             <small>
             This is a new project to automatically compose AI applications that can run across diverse models, data sets, software and hardware
             - please check our presentation at the <a href="https://sites.google.com/g.harvard.edu/mlperf-bench-hpca24/home">MLPerf-Bench workshop @ HPCA'24</a>
             and get in touch via <a href="https://discord.gg/JjWNWXKxwT">Discord</a>!
             </small>
             </i>
              <br>
              <br>
            '''

    else:
        x = '''
             <i>
             <small>
             <a href="https://github.com/mlcommons/cm4mlops/tree/main/script">Collective Mind</a> is a collection of open-source, portable, extensible and ready-to-use
             automation scripts with a human-friendly interface and minimal dependencies to make it easier to compose, benchmark and optimize
             complex AI, ML and other applications and systems across diverse and continuously changing models, data sets, software and hardware.
             Note that this is a <a href="https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md">collaborative engineering effort</a>
             to make sure that they work across all possible versions and configurations
             - please report encountered issues and provide feedback
             <a href="https://github.com/mlcommons/ck/issues">here</a>
             and get in touch via <a href="https://discord.gg/JjWNWXKxwT">Discord</a>!
             </small>
             </i>
              <br>
              <br>
            '''

    st.write(x, unsafe_allow_html=True)

    script_tags = st.text_input(
        'Search open-source automation recipes by tags:',
        value=script_tags,
        key='script_tags').strip()

    # Searching automation recipes

    ii = {'action': 'find',
          'automation': 'script,5b4e0237da074764'}

    if script_tags != '':
        script_tags = script_tags.replace(' ', ',')
        ii['tags'] = script_tags
    elif script_name != '':
        ii['artifact'] = script_name

    # Check variations for later:
    variations = [v for v in script_tags.split(',') if v.startswith('_')]

    r = cmind.access(ii)
    if r['return'] > 0:
        return r

    lst2 = r['list']

    lst = [v for v in lst2 if not v.meta.get('private', False)]

    end_html = ''

    if len(lst) == 0:
        st.markdown('CM scripts were not found!')
    else:
        artifact = None

        if len(lst) == 1:
            # Individual script
            recipe = lst[0]

            meta = recipe.meta

            alias = meta['alias']
            uid = meta['uid']

            use_gui = False
            x = params.get('gui', [''])
            if len(x) > 0 and (x[0].lower() ==
                               'true' or x[0].lower() == 'yes'):
                import script

                script_path = recipe.path
                script_alias = alias

#                script_tags = script_tags
                if script_tags == '':
                    script_tags = meta.get('tags_help', '')
                    if script_tags != '':
                        script_tags = script_tags.replace(' ', ',')
                    else:
                        script_tags = ','.join(meta.get('tags', []))

                ii = {'st': st,
                      'params': params,
                      'script_path': script_path,
                      'script_alias': script_alias,
                      'script_tags': script_tags,
                      'script_meta': meta,
                      'script_repo_meta': recipe.repo_meta,
                      'skip_bottom': True}

                return script.page(ii)

            else:

                st.markdown('### CM script "{}" ({})'.format(alias, uid))

                repo_meta = recipe.repo_meta

                # Basic run
                tags = meta['tags_help'] if meta.get(
                    'tags_help', '') != '' else ' '.join(
                    meta['tags'])

                x1 = misc.make_url(
                    tags.replace(
                        ' ',
                        ','),
                    key='tags',
                    action='scripts',
                    md=False,
                    skip_url_quote=True)
                x2 = misc.make_url(meta['alias'], action='scripts', md=False)
                x3 = misc.make_url(meta['uid'], action='scripts', md=False)
                end_html = '<center><small><i>Self links: <a href="{}">tags</a> or <a href="{}">alias</a> or <a href="{}">UID</a></i></small></center>'.format(
                    x1, x2, x3)

                extra_repo = '' if repo_meta['alias'] == 'mlcommons@cm4mlops' else '\ncm pull repo ' + \
                    repo_meta['alias']

                xtags = tags
                if len(variations) > 0:
                    if xtags != '':
                        xtags += ' '
                    xtags += ' '.join(variations)

                x = '''
```bash
pip install cmind -U
cm test core
cm pull repo mlcommons@cm4mlops --checkout=dev{}

cm run script "{}"
```

A few other popular commands:
```bash
cmr "{}" --help
cmr "{}" --shell
cm run script "{}" --shell
cm docker script "{}"
cm gui script "{}"
```

                    '''.format(extra_repo, xtags, xtags, xtags, xtags, xtags, xtags)

                # Check original link

                url = repo_meta.get('url', '')
                if url == '' and repo_meta.get('git', False):
                    url = 'https://github.com/' + \
                        repo_meta['alias'].replace('@', '/')

                url_readme = ''
                url_readme_extra = ''
                url_meta_description = ''
                url_customize = ''

                if url != '':
                    # Recreate GitHub path
                    if not url.endswith('/'):
                        url = url + '/'

                    url += 'tree/master/'

                    if repo_meta.get('prefix', '') != '':
                        url += repo_meta['prefix']

                    if not url.endswith('/'):
                        url = url + '/'

                    url += 'script/' + alias

                    # Check README.md
                    z = os.path.join(recipe.path, 'README.md')
                    if os.path.isfile(z):
                        url_readme = url + '/README.md'

                    # Check README.extra.md
                    z = os.path.join(recipe.path, 'README-extra.md')
                    if os.path.isfile(z):
                        url_readme_extra = url + '/README-extra.md'

                    # Check customize.py
                    z = os.path.join(recipe.path, 'customize.py')
                    if os.path.isfile(z):
                        url_customize = url + '/customize.py'

                    # Check _cm.yaml or _cm.json
                    for z in ['_cm.yaml', '_cm.json']:
                        y = os.path.join(recipe.path, z)
                        if os.path.isfile(y):
                            url_meta_description = url + '/' + z

                url_gui = url_prefix_script + '&name=' + alias + ',' + uid + '&gui=true'

                z = '* ***Check [open source code (Apache 2.0 license)]({}) at GitHub.***\n'.format(
                    url)
                z += '* ***Check [detailed auto-generated README on GitHub]({}).***\n'.format(
                    url_readme)
                z += '* ***Check [experimental GUI]({}) to run this script.***\n'.format(
                    url_gui)
                z += '---\n'

                st.markdown(z)

                st.markdown(
                    'Default run on Linux, Windows, MacOS and any other OS (check [CM installation guide]({}) for more details):\n{}\n'.format(
                        url_prefix +
                        '?action=install',
                        x))

                st.markdown('*The [Collective Mind concept](https://doi.org/10.5281/zenodo.8105339) is to gradually improve portability and reproducibility of common automation recipes based on user feedback'
                            ' while keeping the same human-friendly interface. If you encounter issues, please report them [here](https://github.com/mlcommons/ck/issues) '
                            ' to help this community project!*')

                if url_readme_extra != '':
                    st.markdown(
                        '* See [extra README]({}) for this automation recipe at GitHub.'.format(url_readme_extra))

                if url_meta_description != '':
                    st.markdown(
                        '* See [meta description]({}) for this automation recipe at GitHub.'.format(url_meta_description))

                if url_customize != '':
                    st.markdown(
                        '* See [customization python code]({}) for this automation recipe at GitHub.'.format(url_customize))

                # Check dependencies
                r = misc.get_all_deps_tags({'meta': meta, 'st': st})
                if r['return'] > 0:
                    return r

                all_deps_tags = r['all_deps_tags']

                if len(all_deps_tags) > 0:
                    st.markdown('**Dependencies on other CM scripts:**')

                    x = ''
                    for t in sorted(all_deps_tags):
                        # Test that it's not just extending tags:
                        if t.startswith('_') or ',' not in t:
                            continue

                        url_deps = url_prefix_script + '&tags=' + t

                        x += '* [{}]({})\n'.format(t, url_deps)

                    st.markdown(x)

        else:
            categories = {}

            for l in sorted(lst, key=lambda x: (
                x.meta.get('alias', '')
            )):

                category = l.meta.get('category', '')
                if category == '':
                    category = 'Unsorted'

                if category not in categories:
                    categories[category] = []

                categories[category].append(l)

            if len(categories) > 1:
                category_selection = [
                    ''] + sorted(list(categories.keys()), key=lambda v: v.upper())

                # Creating compute selector
                category_id = st.selectbox('Prune by category:',
                                           range(len(category_selection)),
                                           format_func=lambda x: category_selection[x],
                                           index=0,
                                           key='category')

                if category_id > 0:
                    category_key = category_selection[category_id]
                    categories = {category_key: categories[category_key]}

            # Check number of recipes
            recipes = 0
            for category in sorted(categories, key=lambda v: v.upper()):
                recipes += len(categories[category])

            x = '''
                <small>
                 Found {} automation recipes:
                </center>
                '''.format(str(recipes))
            st.write(x, unsafe_allow_html=True)

            for category in sorted(categories, key=lambda v: v.upper()):
                md = '### {}'.format(category) + '\n'

                for recipe in categories[category]:
                    meta = recipe.meta

                    alias = meta['alias']
                    uid = meta['uid']

                    url = url_prefix_script + '&name=' + alias + ',' + uid

                    md += '* [{}]({})'.format(alias, url) + '\n'

                st.markdown(md)

    return {'return': 0, 'end_html': end_html}



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import os
import tarfile


def preprocess(i):

    recursion_spaces = i['recursion_spaces']

    os_info = i['os_info']

    env = i['env']

    # Not enforcing dev requirement for now
    if env.get('CM_TENSORRT_TAR_FILE_PATH', '') == '' and env.get(
            'CM_TENSORRT_REQUIRE_DEV1', '') != 'yes' and env.get('CM_HOST_PLATFORM_FLAVOR_', '') != 'aarch64':

        if os_info['platform'] == 'windows':
            extra_pre = ''
            extra_ext = 'lib'
        else:
            extra_pre = 'lib'
            extra_ext = 'so'

        libfilename = extra_pre + 'nvinfer.' + extra_ext
        env['CM_TENSORRT_VERSION'] = 'vdetected'

        if env.get('CM_TMP_PATH', '').strip() != '':
            path = env.get('CM_TMP_PATH')
            if os.path.exists(os.path.join(path, libfilename)):
                env['CM_TENSORRT_LIB_PATH'] = path
                return {'return': 0}

        if not env.get('CM_TMP_PATH'):
            env['CM_TMP_PATH'] = ''

        if os_info['platform'] == 'windows':
            if env.get('CM_INPUT', '').strip() == '' and env.get(
                    'CM_TMP_PATH', '').strip() == '':
                # Check in "C:\Program Files\NVIDIA GPU Computing Toolkit\CUDA"
                paths = []
                for path in ["C:\\Program Files\\NVIDIA GPU Computing Toolkit\\CUDA",
                             "C:\\Program Files (x86)\\NVIDIA GPU Computing Toolkit\\CUDA"]:
                    if os.path.isdir(path):
                        dirs = os.listdir(path)
                        for dr in dirs:
                            path2 = os.path.join(path, dr, 'lib')
                            if os.path.isdir(path2):
                                paths.append(path2)

                if len(paths) > 0:
                    tmp_paths = ';'.join(paths)
                    tmp_paths += ';' + os.environ.get('PATH', '')

                    env['CM_TMP_PATH'] = tmp_paths
                    env['CM_TMP_PATH_IGNORE_NON_EXISTANT'] = 'yes'

        else:
            # paths to cuda are not always in PATH - add a few typical locations to search for
            # (unless forced by a user)

            if env.get('CM_INPUT', '').strip() == '':
                if env.get('CM_TMP_PATH', '').strip() != '':
                    env['CM_TMP_PATH'] += ':'

                env['CM_TMP_PATH_IGNORE_NON_EXISTANT'] = 'yes'

                for lib_path in env.get(
                        '+CM_HOST_OS_DEFAULT_LIBRARY_PATH', []):
                    if (os.path.exists(lib_path)):
                        env['CM_TMP_PATH'] += ':' + lib_path

        r = i['automation'].find_artifact({'file_name': libfilename,
                                           'env': env,
                                           'os_info': os_info,
                                           'default_path_env_key': 'LD_LIBRARY_PATH',
                                           'detect_version': False,
                                           'env_path_key': 'CM_TENSORRT_LIB_WITH_PATH',
                                           'run_script_input': i['run_script_input'],
                                           'recursion_spaces': recursion_spaces})
        if r['return'] > 0:
            if os_info['platform'] == 'windows':
                return r
        else:
            return {'return': 0}

    if os_info['platform'] == 'windows':
        return {'return': 1, 'error': 'Windows is currently not supported!'}

    if env.get('CM_TENSORRT_TAR_FILE_PATH', '') == '':
        tags = ["get", "tensorrt"]
        if env.get('CM_TENSORRT_REQUIRE_DEV', '') != 'yes':
            tags.append("_dev")
        return {'return': 1, 'error': 'Please envoke cmr "' +
                " ".join(tags) + '" --tar_file={full path to the TensorRT tar file}'}

    print('Untaring file - can take some time ...')

    file_name = "trtexec"
    my_tar = tarfile.open(os.path.expanduser(env['CM_TENSORRT_TAR_FILE_PATH']))
    folder_name = my_tar.getnames()[0]
    if not os.path.exists(os.path.join(os.getcwd(), folder_name)):
        my_tar.extractall()
    my_tar.close()

    import re
    version_match = re.match(r'TensorRT-(\d+\.\d+\.\d+\.\d+)', folder_name)
    if not version_match:
        return {'return': 1, 'error': 'Extracted TensorRT folder does not seem proper - Version information missing'}
    version = version_match.group(1)

    env['CM_TENSORRT_VERSION'] = version
    env['CM_TENSORRT_INSTALL_PATH'] = os.path.join(os.getcwd(), folder_name)
    env['CM_TENSORRT_LIB_PATH'] = os.path.join(os.getcwd(), folder_name, "lib")
    env['CM_TMP_PATH'] = os.path.join(os.getcwd(), folder_name, "bin")
    env['+CPLUS_INCLUDE_PATH'] = [
        os.path.join(
            os.getcwd(),
            folder_name,
            "include")]
    env['+C_INCLUDE_PATH'] = [
        os.path.join(
            os.getcwd(),
            folder_name,
            "include")]
    env['+LD_LIBRARY_PATH'] = [os.path.join(os.getcwd(), folder_name, "lib")]

    return {'return': 0}


def postprocess(i):

    os_info = i['os_info']

    env = i['env']

    if '+LD_LIBRARY_PATH' not in env:
        env['+LD_LIBRARY_PATH'] = []

    if '+PATH' not in env:
        env['+PATH'] = []

    if '+ LDFLAGS' not in env:
        env['+ LDFLAGS'] = []

    # if 'CM_TENSORRT_LIB_WITH_PATH' in env:
    #    tensorrt_lib_path = os.path.dirname(env['CM_TENSORRT_LIB_WITH_PATH'])
    if 'CM_TENSORRT_LIB_PATH' in env:
        env['+LD_LIBRARY_PATH'].append(env['CM_TENSORRT_LIB_PATH'])
        env['+PATH'].append(env['CM_TENSORRT_LIB_PATH'])  # for cmake
        env['+ LDFLAGS'].append("-L" + env['CM_TENSORRT_LIB_PATH'])

    version = env['CM_TENSORRT_VERSION']

    return {'return': 0, 'version': version}



import json
import sys
import os

with open(sys.argv[1], "r") as f:
    data = json.load(f)

images = {}
for image in data['images']:
    images[image['id']] = image
    images[image['id']]['num_boxes'] = 0

annots = data['annotations']
for box in annots:
    imageid = box['image_id']
    images[imageid]['num_boxes'] += 1

sorted_image_data = sorted(
    data['images'],
    key=lambda x: x['num_boxes'],
    reverse=os.environ.get(
        'CM_CALIBRATION_FILTER_ORDER_BY_NUM_BOXES_ASC',
        '') == "yes")
for image in data['images']:
    print(image['file_name'])



"""
Developers:
 - grigori@octoml.ai
"""

import time
import os
import argparse
import json

from PIL import Image
import cv2

import numpy as np

import onnxruntime as rt


# Image conversion from MLPerf(tm) vision
def center_crop(img, out_height, out_width):
    height, width, _ = img.shape
    left = int((width - out_width) / 2)
    right = int((width + out_width) / 2)
    top = int((height - out_height) / 2)
    bottom = int((height + out_height) / 2)
    img = img[top:bottom, left:right]
    return img


def resize_with_aspectratio(
        img, out_height, out_width, scale=87.5, inter_pol=cv2.INTER_LINEAR):
    height, width, _ = img.shape
    new_height = int(100. * out_height / scale)
    new_width = int(100. * out_width / scale)
    if height > width:
        w = new_width
        h = int(new_height * height / width)
    else:
        h = new_height
        w = int(new_width * width / height)
    img = cv2.resize(img, (w, h), interpolation=inter_pol)
    return img


# returns list of pairs (prob, class_index)
def get_top5(all_probs):
    probs_with_classes = []

    for class_index in range(len(all_probs)):
        prob = all_probs[class_index]
        probs_with_classes.append((prob, class_index))

    sorted_probs = sorted(
        probs_with_classes,
        key=lambda pair: pair[0],
        reverse=True)
    return sorted_probs[0:5]


def run_case(dtype, image, target):
    # Check image
    import os
    import json
    import sys

    STAT_REPEAT = os.environ.get('STAT_REPEAT', '')
    if STAT_REPEAT == '' or STAT_REPEAT is None:
        STAT_REPEAT = 10
    STAT_REPEAT = int(STAT_REPEAT)

    # FGG: set model files via CM env
    CATEG_FILE = 'synset.txt'
    synset = eval(open(os.path.join(CATEG_FILE)).read())

    files = []
    val = {}

    # FGG: set timers
    import time
    timers = {}

    img_orig = cv2.imread(image)

    img = cv2.cvtColor(img_orig, cv2.COLOR_BGR2RGB)

    output_height, output_width, _ = 224, 224, 3
    img = resize_with_aspectratio(
        img,
        output_height,
        output_width,
        inter_pol=cv2.INTER_AREA)
    img = center_crop(img, output_height, output_width)
    img = np.asarray(img, dtype='float32')

    # normalize image
    means = np.array([123.68, 116.78, 103.94], dtype=np.float32)
    img -= means

    # transpose if needed
    img = img.transpose([2, 0, 1])

    import matplotlib.pyplot as plt
    img1 = img.transpose([1, 2, 0])
    # you can give axis attribute if you wanna squeeze in specific dimension
    arr_ = np.squeeze(img1)
    plt.imshow(arr_)
#    plt.show()
    plt.savefig('pre-processed-image.png')
    # Load model
    model_path = os.environ.get('CM_ML_MODEL_FILE_WITH_PATH', '')
    if model_path == '':
        print('Error: environment variable CM_ML_MODEL_FILE_WITH_PATH is not defined')
        exit(1)

    opt = rt.SessionOptions()

    if len(rt.get_all_providers()) > 1 and os.environ.get(
            "USE_CUDA", "yes").lower() not in ["0", "false", "off", "no"]:
        # Currently considering only CUDAExecutionProvider
        sess = rt.InferenceSession(
            model_path, opt, providers=['CUDAExecutionProvider'])
    else:
        sess = rt.InferenceSession(
            model_path, opt, providers=["CPUExecutionProvider"])

    inputs = [meta.name for meta in sess.get_inputs()]
    outputs = [meta.name for meta in sess.get_outputs()]

    print(inputs)
    print(outputs)

    if os.environ.get('USE_TVM', '') == 'yes':
        import tvm
        from tvm import relay
        import onnx

        del sess

        # Load model via ONNX to be used with TVM
        print('')
        print('ONNX: load model ...')
        print('')

        onnx_model = onnx.load(model_path)

        # Init TVM
        # TBD: add tvm platform selector
        if os.environ.get('USE_CUDA', '') == 'yes':
            # TVM package must be built with CUDA enabled
            ctx = tvm.cuda(0)
        else:
            ctx = tvm.cpu(0)
        tvm_ctx = ctx

        build_conf = {'relay.backend.use_auto_scheduler': False}
        opt_lvl = int(os.environ.get('TVM_OPT_LEVEL', 3))
        host = os.environ.get('CM_HOST_PLATFORM_FLAVOR')
        if host == 'x86_64' and 'AMD' in os.environ.get(
                'CM_HOST_CPU_VENDOR_ID', ''):
            target = os.environ.get('TVM_TARGET', 'llvm -mcpu=znver2')
        else:
            target = os.environ.get('TVM_TARGET', 'llvm')

        target_host = None
        params = {}

        # New target API
        tvm_target = tvm.target.Target(target, host=target_host)

        input_shape = (1, 3, 224, 224)
        shape_dict = {inputs[0]: input_shape}

        print('')
        print('TVM: import model ...')
        print('')
        # Extra param: opset=12
        mod, params = relay.frontend.from_onnx(
            onnx_model, shape_dict, freeze_params=True)

        print('')
        print('TVM: transform to static ...')
        print('')
        mod = relay.transform.DynamicToStatic()(mod)

        print('')
        print('TVM: apply extra optimizations ...')
        print('')
        # Padding optimization
        # Adds extra optimizations
        mod = relay.transform.FoldExplicitPadding()(mod)

        print('')
        print('TVM: build model ...')
        print('')

        executor = os.environ.get('MLPERF_TVM_EXECUTOR', 'graph')

        if executor == "graph" or executor == "debug":
            from tvm.contrib import graph_executor

            # Without history
            with tvm.transform.PassContext(opt_level=opt_lvl, config=build_conf):
                graph_module = relay.build(mod,
                                           target=tvm_target,
                                           params=params)
            lib = graph_module

            print('')
            print('TVM: init graph engine ...')
            print('')

            sess = graph_executor.GraphModule(lib['default'](ctx))

        elif executor == "vm":
            from tvm.runtime.vm import VirtualMachine

            # Without history
            with tvm.transform.PassContext(opt_level=opt_lvl, config=build_conf):
                vm_exec = relay.vm.compile(
                    mod, target=tvm_target, params=params)

            r_exec = vm_exec

            print('')
            print('TVM: init VM ...')
            print('')

            sess = VirtualMachine(r_exec, ctx)

        # For now only graph
        sess.set_input(inputs[0], tvm.nd.array([img]))

        # Run TVM inference
        sess.run()

        # Process TVM outputs
        output = []

        for i in range(sess.get_num_outputs()):
            # Take only the output of batch size for dynamic batches
            if len(output) < (i + 1):
                output.append([])
            output[i].append(sess.get_output(i).asnumpy()[0])

    else:
        inp = {inputs[0]: np.array([img], dtype=np.float32)}
        output = sess.run(outputs, inp)

    top1 = np.argmax(output[1]) - 1  # .asnumpy())

    top5 = []
    atop5 = get_top5(output[1][0])  # .asnumpy())

    print('')
    print('Prediction Top1:', top1, synset[top1])

    print('')
    print('Prediction Top5:')
    for p in atop5:
        out = p[1] - 1
        name = synset[out]
        print(' * {} {}'.format(out, name))

    ck_results = {
        'prediction': synset[top1]
    }

    with open('tmp-ck-timer.json', 'w') as ck_results_file:
        json.dump(ck_results, ck_results_file, indent=2, sort_keys=True)

    return


if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument(
        '--image',
        type=str,
        help="Path to JPEG image.",
        default=None,
        required=True)
    parser.add_argument('--target', type=str, help="Target", default=None)
    args = parser.parse_args()

    if args.image.strip().lower() == '':
        print('Please specify path to an image using CM_IMAGE environment variable!')
        exit(1)

    # set parameter
    batch_size = 1
    num_classes = 1000
    image_shape = (3, 224, 224)

    # load model
    data_shape = (batch_size,) + image_shape
    out_shape = (batch_size, num_classes)

    dtype = 'float32'
    if os.environ.get('CM_TVM_DTYPE', '') != '':
        dtype = os.environ['CM_TVM_DTYPE']

    run_case(dtype, args.image, args.target)



import bcrypt

# salt = bcrypt.gensalt()
# TBD: temporal hack to demo password protection for experiments
# salt = bcrypt.gensalt()

pwd = input('Password: ')
pwd = pwd.strip()

password_salt = b'$2b$12$ionIRWe5Ft7jkn4y/7C6/e'
password_hash2 = bcrypt.hashpw(pwd.encode('utf-8'), password_salt)

print('"password_hash":"{}"'.format(password_hash2.decode('utf-8')))



import sys
import os
import submission_checker as checker
from log_parser import MLPerfLog


def get_result_from_log(version, model, scenario,
                        result_path, mode, inference_src_version=None):

    config = checker.Config(
        version,
        None,
        ignore_uncommited=False,
        skip_power_check=False,
    )
    mlperf_model = config.get_mlperf_model(model)
    # scenario = checker.SCENARIO_MAPPING[scenario]

    result = ''
    power_result = None
    valid = {}
    if mode == "performance":
        has_power = os.path.exists(os.path.join(result_path, "..", "power"))
        version_tuple = None
        if inference_src_version:
            version_tuple = tuple(map(int, inference_src_version.split('.')))

        if version_tuple and version_tuple >= (4, 1, 22):
            result_ = checker.get_performance_metric(
                config, mlperf_model, result_path, scenario)
        else:
            result_ = checker.get_performance_metric(
                config, mlperf_model, result_path, scenario, None, None, has_power)
        mlperf_log = MLPerfLog(
            os.path.join(
                result_path,
                "mlperf_log_detail.txt"))
        if (
            "result_validity" not in mlperf_log.get_keys()
            or mlperf_log["result_validity"] != "VALID"
        ):
            valid['performance'] = False
        else:
            valid['performance'] = True

        if "stream" in scenario.lower():
            result = result_ / 1000000  # convert to milliseconds
        else:
            result = result_
        result = str(round(result, 3))

        if has_power:
            power_valid, power_metric, scenario, avg_power_efficiency = checker.get_power_metric(
                config, scenario, result_path, True, result_)
            power_result = f"{round(power_metric,3)},{round(avg_power_efficiency,3)}"
            valid['power'] = power_valid

    elif mode == "accuracy" and os.path.exists(os.path.join(result_path, 'accuracy.txt')):

        acc_valid, acc_results, acc_targets, acc_limits = get_accuracy_metric(
            config, mlperf_model, result_path)
        valid['accuracy'] = acc_valid

        if len(acc_results) == 1:
            for acc in acc_results:
                result = str(round(float(acc_results[acc]), 5))
        else:
            result = '('
            result_list = []
            for i, acc in enumerate(acc_results):
                result_list.append(str(round(float(acc_results[acc]), 5)))
            result += ", ".join(result_list) + ")"

    return result, valid, power_result


def get_accuracy_metric(config, model, path):

    import re
    is_valid = False
    all_accuracy_valid = True
    acc = None
    result_acc = None
    target = config.get_accuracy_target(model)
    acc_upper_limit = config.get_accuracy_upper_limit(model)
    patterns = []
    acc_targets = []
    acc_limits = [None] * (len(target) // 2)
    up_patterns = [None] * (len(target) // 2)
    acc_types = []

    if acc_upper_limit is not None:
        acc_limit_check = True

        for ii in range(0, len(target), 2):
            acc_type1, tmp = target[ii:ii + 2]
            for i in range(0, len(acc_upper_limit), 2):
                acc_type, acc_target = acc_upper_limit[i:i + 2]
                if acc_type != acc_type1:
                    continue
                acc_limits[ii // 2] = acc_target
                up_patterns[ii // 2] = checker.ACC_PATTERN[acc_type]

    for i in range(0, len(target), 2):
        acc_type, acc_target = target[i:i + 2]
        acc_types.append(acc_type)
        patterns.append(checker.ACC_PATTERN[acc_type])
        acc_targets.append(acc_target)

    acc_seen = [False for _ in acc_targets]
    acc_results = {}
    with open(os.path.join(path, "accuracy.txt"), "r", encoding="utf-8") as f:
        for line in f:
            for i, (pattern, acc_target, acc_type) in enumerate(
                    zip(patterns, acc_targets, acc_types)):
                m = re.match(pattern, line)
                if m:
                    acc = m.group(1)

                    acc_results[acc_type] = acc

                if acc is not None and float(acc) >= acc_target:
                    all_accuracy_valid &= True
                    acc_seen[i] = True
                elif acc is not None:
                    all_accuracy_valid = False
                    # log.warning("%s accuracy not met: expected=%f, found=%s", path, acc_target, acc)
                if i == 0 and acc:
                    result_acc = acc
                acc = None
            if acc_upper_limit is not None:
                for i, (pattern, acc_limit) in enumerate(
                        zip(up_patterns, acc_limits)):
                    if not pattern:
                        continue
                    m = re.match(pattern, line)
                    if m:
                        acc = m.group(1)
                    if acc is not None and acc_upper_limit is not None and float(
                            acc) > acc_limit:
                        acc_limit_check = False
                        # log.warning("%s accuracy not met: upper limit=%f, found=%s", path, acc_limit, acc)
                    acc = None
            if all(acc_seen):
                break
        is_valid = all_accuracy_valid & all(acc_seen)
        if acc_upper_limit is not None:
            is_valid &= acc_limit_check

    return is_valid, acc_results, acc_targets, acc_limits


def get_result_string(version, model, scenario, result_path, has_power, sub_res,
                      division="open", system_json=None, model_precision="fp32", inference_src_version=None):

    config = checker.Config(
        version,
        None,
        ignore_uncommited=False,
        skip_power_check=False,
    )
    mlperf_model = config.get_mlperf_model(model)
    performance_path = os.path.join(result_path, "performance", "run_1")
    accuracy_path = os.path.join(result_path, "accuracy")
    scenario = checker.SCENARIO_MAPPING[scenario.lower()]

    fname = os.path.join(performance_path, "mlperf_log_detail.txt")
    mlperf_log = MLPerfLog(fname)
    effective_scenario = mlperf_log["effective_scenario"]
    inferred = False
    result = {}

    version_tuple = None
    if inference_src_version:
        version_tuple = tuple(map(int, inference_src_version.split('.')))

    if version_tuple and version_tuple >= (4, 1, 22):
        performance_result = checker.get_performance_metric(
            config, mlperf_model, performance_path, scenario)
    else:
        performance_result = checker.get_performance_metric(
            config, mlperf_model, performance_path, scenario, None, None)
    if "stream" in scenario.lower():
        performance_result_ = performance_result / 1000000  # convert to milliseconds
    else:
        performance_result_ = performance_result
    result['performance'] = round(performance_result_, 3)

    if scenario != effective_scenario:
        inferred, inferred_result = checker.get_inferred_result(
            scenario, effective_scenario, performance_result, mlperf_log, config, False)

    if has_power:
        is_valid, power_metric, scenario, avg_power_efficiency = checker.get_power_metric(
            config, scenario, performance_path, True, performance_result)
        if "stream" in scenario.lower():
            power_metric_unit = "milliJoules"
        else:
            power_metric_unit = "Watts"
        power_result_string = f"`Power consumed`: `{round(power_metric, 3)} {power_metric_unit}`, `Power efficiency`: `{round(avg_power_efficiency * 1000, 3)} samples per Joule`"

        power_result = round(power_metric, 3)
        power_efficiency_result = round(avg_power_efficiency, 3)
        result['power'] = power_result
        result['power_efficiency'] = power_efficiency_result

    compliance_list = ["TEST01", "TEST04", "TEST06"]
    if division == "closed":
        for test in compliance_list:
            test_path = os.path.join(result_path, test)
            if os.path.exists(
                    test_path):  # We dont consider missing test folders now - submission checker will do that
                # test_pass = checker.check_compliance_dir(test_path, mlperf_model, scenario, config, "closed", system_json, sub_res)
                test_pass = checker.check_compliance_perf_dir(
                    test_path) if test != "TEST06" else True
                if test_pass and test in ["TEST01", "TEST06"]:
                    # test_pass = checker.check_compliance_acc_dir(test_path, mlperf_model, config)
                    pass  # accuracy truncation script is done after submission generation. We assume here that it'll pass
                if test_pass:
                    result[test] = "passed"
                else:
                    result[test] = "failed"

    acc_valid, acc_results, acc_targets, acc_limits = get_accuracy_metric(
        config, mlperf_model, accuracy_path)

    result_field = checker.RESULT_FIELD[effective_scenario]

    performance_result_string = f"`{result_field}`: `{performance_result}`\n"
    if inferred:
        inferred_result_field = checker.RESULT_FIELD[scenario]
        performance_result_string += f"Inferred result: `{inferred_result_field}`: `{inferred_result}`  \n"

    accuracy_result_string = ''
    accuracy_results = []
    for i, acc in enumerate(acc_results):
        accuracy_results.append(str(round(float(acc_results[acc]), 5)))
        accuracy_result_string += f"`{acc}`: `{round(float(acc_results[acc]), 5)}`"
        if not acc_limits or not acc_limits[i]:
            accuracy_result_string += f", Required accuracy for closed division `>= {round(acc_targets[i], 5)}`"
        else:
            accuracy_result_string += f", Required accuracy for closed division `>= {round(acc_targets[i], 5)}` and `<= {round(acc_limits[i], 5)}`"
        accuracy_result_string += "\n"

    if len(accuracy_results) == 1:
        accuracy_result = accuracy_results[0]
    else:
        accuracy_result = "(" + ", ".join(accuracy_results) + ")"
    result['accuracy'] = accuracy_result

    result_string = f"\n\n## Results\n"
    result_string += f"\nPlatform: {sub_res}\n"
    result_string += f"\nModel Precision: {model_precision}\n"
    result_string += "\n### Accuracy Results \n" + accuracy_result_string
    result_string += "\n### Performance Results \n" + performance_result_string
    if has_power:
        result_string += "\n### Power Results \n" + power_result_string

    return result_string, result


def get_result_table(results):

    headers = [
        "Model",
        "Scenario",
        "Accuracy",
        "Throughput",
        "Latency (in ms)",
        "Power Efficiency (in samples/J)",
        "TEST01",
        "TEST04"]
    table = []
    for model in results:
        for scenario in results[model]:
            row = []
            row.append(model)
            row.append(scenario)
            if results[model][scenario].get('accuracy'):
                val = str(results[model][scenario]['accuracy'])
                if not results[model][scenario].get('accuracy_valid', True):
                    val = "X " + val
                row.append(val)
            else:
                row.append("-")

            if results[model][scenario].get('performance'):

                if "stream" in scenario.lower():
                    if float(results[model][scenario]['performance']) == 0:
                        row.append("-")
                    elif scenario.lower() == "singlestream":
                        val_qps = str(
                            round(
                                1000 /
                                float(
                                    results[model][scenario]['performance']),
                                3))
                        if not results[model][scenario].get(
                                'performance_valid', True):  # we explicitly mark invalid results
                            val_qps = "X " + val_qps
                        row.append(val_qps)
                    elif scenario.lower() == "multistream":
                        val_qps = str(
                            round(
                                8000 /
                                float(
                                    results[model][scenario]['performance']),
                                3))
                        if not results[model][scenario].get(
                                'performance_valid', True):
                            val_qps = "X " + val_qps
                        row.append(val_qps)
                    val = str(results[model][scenario]['performance'])
                    if not results[model][scenario].get(
                            'performance_valid', True):
                        val = "X " + val
                    row.append(val)
                else:
                    val = str(results[model][scenario]['performance'])
                    if not results[model][scenario].get(
                            'performance_valid', True):
                        val = "X " + val
                    row.append(val)
                    row.append("-")

            val1 = results[model][scenario].get('TEST01')
            # val2 = results[model][scenario].get('TEST05')
            val3 = results[model][scenario].get('TEST04')

            # if results[model][scenario].get('power','') != '':
            #    row.append(results[model][scenario]['power'])
            if results[model][scenario].get('power_efficiency', '') != '':
                val = str(results[model][scenario]['power_efficiency'])
                if not results[model][scenario].get('power_valid', True):
                    val = "X " + val
                row.append(val)
            elif val1 or val3:  # Don't output unless there are any further column data
                row.append(None)

            if val1:
                row.append(val1)
                if val3:
                    row.append(val3)

            else:
                if val3:
                    row.append("missing")
                    row.append(val3)

            table.append(row)

    return table, headers



# Support functions

##########################################################
def make_url(name, alias='', action='contributors',
             key='name', md=True, skip_url_quote=False):

    import urllib

    if alias == '':
        alias = name

    x = urllib.parse.quote_plus(alias) if not skip_url_quote else alias

    xaction = ''
    if action != '':
        xaction = 'action={}'.format(action)
        if key != '':
            xaction += '&'

    url = '?{}'.format(xaction)

    if key != '':
        url += '{}={}'.format(key, x)

    if md:
        md = '[{}]({})'.format(name, url)
    else:
        md = url

    return md

##########################################################


def convert_date(date):
    # date: format YYYYMMDD to YYYY month day

    import calendar

    try:
        year = date[0:4]
        month = calendar.month_abbr[int(date[4:6])]
        day = str(int(date[6:8]))
    except Exception as e:
        return {'return': 1, 'error': 'date "{}" is not of format YYYYMMDD: {}'.format(
            date, format(e))}

    return {'return': 0, 'string': year + ' ' + month + ' ' + day}

##########################################################


def get_params(st):
    compatibility = False

    try:
        params2 = st.query_params
        # Convert to old style
        params = {}
        for k in params2:
            v = params2[k]
            if type(v) != list:
                params[k] = [v]
    except BaseException:
        compatibility = True

    if compatibility:
        params = st.experimental_get_query_params()

    return params

##########################################################


def get_all_deps_tags(i):
    meta = i['meta']
    all_deps_tags = i.get('all_deps_tags', [])

    for k in meta:
        v = meta[k]

        if k == 'tags':


if isinstance(v,             if)                v = ','.join(v)

            if v not in all_deps_tags:
                all_deps_tags.append(v)

elif isinstance(v,         elif )            r = get_all_deps_tags({'meta': v, 'all_deps_tags': all_deps_tags})
            all_deps_tags = r['all_deps_tags']

elif isinstance(v,         elif)            for vv in v:
if isinstance(vv,                 if )                    r = get_all_deps_tags({'meta': vv, 'all_deps_tags': all_deps_tags})
                    all_deps_tags = r['all_deps_tags']

    return {'return': 0, 'all_deps_tags':all_deps_tags}

##########################################################

def make_selector(i):

    key = i['key']
    value = i['desc']

    params = i['params']

    st = i['st']
    st_inputs = i['st_inputs']

    hide = i.get('hide', False)

    key2 = '@' +key

    value2 = None

if isinstance(value,     if )        desc = value['desc']

        choices = value.get('choices', [])
        boolean = value.get('boolean', False)
        default = value.get('default', '')
        force = value.get('force', None)

        if force is not None:
            value2 = force
            if not hide:
                st.markdown('**{}:** {}'.format(desc, str(force)))

        else:
            if boolean:
                v = default
                x = params.get(key2, None)
                if x !=None and len(x)>0 and x[0]!=None:
                    if x[0].lower() =='true':
                        v = True
                    elif x[0].lower() =='false':
                        v = False
                if hide:
                    value2 = v
                else:
                    value2 = st.checkbox(desc, value=v, key=key2)
            elif len(choices) >0:
                x = params.get(key2, None)
                if x !=None and len(x)>0 and x[0]!=None:
                    x = x[0]
                    if x in choices:
                        selected_index = choices.index(
                            x) if x in choices else 0
                    else:
                        selected_index = choices.index(default) if default != '' else 0
                else:
                    selected_index = choices.index(default) if default != '' else 0
                if hide:
                    value2 = choices[selected_index]
                else:
                    value2 = st.selectbox(
    desc, choices, index=selected_index, key=key2)
            else:
                v = default
                x = params.get(key2, None)
                if x !=None and len(x)>0 and x[0]!=None:
                    v = x[0]
                if hide:
                    value2 = v
                else:
                    value2 = st.text_input(desc, value=v, key=key2)

        st_inputs[key2] = value2

    else:
        desc = value
        if hide:
            value2 = desc
        else:
            value2 = st.text_input(desc)
        st_inputs[key2] = value2

    return {'return': 0, 'key2': key2, 'value2': value2}

##########################################################

def make_selection(st, selection, param_key, text, x_uid, force_index=0):

    x_meta = {}

    if len(selection) >0:
        selection = sorted(selection, key= lambda v: v['name'])

        if x_uid != '':
            x_meta = selection[0]
            st.markdown('**Selected {}:** {}'.format(text, x_meta['name']))
        else:
            x_selection = [{'name': ''}]
            x_selection += selection

            x_id = st.selectbox('Select {}:'.format(text),
                                range(len(x_selection)),
                                format_func=lambda x: x_selection[x]['name'],
                                index= force_index,
                                key= param_key)

            if x_id >0:
                x_meta = x_selection[x_id]

    return {'return': 0, 'meta':x_meta}

##########################################################################

def get_with_complex_key_safe(meta, key):
    v = get_with_complex_key(meta, key)

    if v == None:
        v=''

    return v

##########################################################################

def get_with_complex_key(meta, key):

    j = key.find('.')

    if j <0:
        return meta.get(key)

    key0 = key[:j]

    if key0 not in meta:
        return None

    return get_with_complex_key(meta[key0], key[j +1:])



import cmind
import os
import pathlib
current_file_path = pathlib.Path(__file__).parent.resolve()
docker_os = {
    "ubuntu": ["18.04", "20.04", "22.04"],
    "rhel": ["9"]
}
dataset = {
    "resnet50": "imagenet",
    "retinanet": "openimages",
    "bert-99.9": "squad"
}
variations = {
    "resnet50": {
        "tensorflow": {
            "cpu": ["python"]
        },
        "onnxruntime": {
            "cpu": ["python", "cpp"]
        },
        "pytorch": {
            "cpu": []
        }
    },
    "retinanet": {
        "tensorflow": {
        },
        "onnxruntime": {
            "cpu": ["python", "cpp"]
        },
        "pytorch": {
            "cpu": ["python"]
        }
    },
    "bert-99.9": {
        "tensorflow": {
            "cpu": ["python"]
        },
        "onnxruntime": {
            "cpu": ["python"]
        },
        "pytorch": {
            "cpu": []
        }
    }
}

for _os in docker_os:
    for version in docker_os[_os]:
        for model in variations:
            for backend in variations[model]:
                for device in variations[model][backend]:
                    for implementation in variations[model][backend][device]:
                        variation_string = ",_" + model + ",_" + \
                            backend + ",_" + device + ",_" + implementation
                        file_name_ext = "_" + implementation + "_" + backend + "_" + device
                        dockerfile_path = os.path.join(
                            current_file_path,
                            'dockerfiles',
                            model,
                            _os +
                            '_' +
                            version +
                            file_name_ext +
                            '.Dockerfile')
                        cm_input = {'action': 'run',
                                    'automation': 'script',
                                    'tags': 'app,mlperf,inference,generic' + variation_string,
                                    'adr': {'compiler':
                                            {'tags': 'gcc'},
                                            'inference-src':
                                            {'tags': '_octoml'},
                                            'openimages-preprocessed':
                                            {'tags': '_50'}
                                            },
                                    'print_deps': True,
                                    'quiet': True,
                                    'silent': True,
                                    'fake_run': True
                                    }
                        r = cmind.access(cm_input)
                        print_deps = r['new_state']['print_deps']
                        comments = ["#RUN " + dep for dep in print_deps]
                        comments.append("")
                        comments.append(
                            "# Run CM workflow for MLPerf inference")
                        cm_docker_input = {'action': 'run',
                                           'automation': 'script',
                                           'tags': 'build,dockerfile',
                                           'docker_os': _os,
                                           'docker_os_version': version,
                                           'file_path': dockerfile_path,
                                           'comments': comments,
                                           'run_cmd': 'cm run script --tags=app,mlperf,inference,generic' + variation_string + ' --adr.compiler.tags=gcc --adr.inference-src.tags=_octoml',
                                           'script_tags': 'app,mlperf,inference,generic',
                                           'quiet': True,
                                           'print_deps': True,
                                           'real_run': True
                                           }
                        r = cmind.access(cm_docker_input)
                        if r['return'] > 0:
                            print(r)
                            exit(1)

                        print('')
                        print("Dockerfile generated at " + dockerfile_path)



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import os
from os.path import exists
import shutil
import glob


def preprocess(i):

    env = i['env']
    if 'CM_IMAGENET_PREPROCESSED_PATH' in env:
        files = glob.glob(
            env['CM_IMAGENET_PREPROCESSED_PATH'] +
            "/**/" +
            env['CM_IMAGENET_PREPROCESSED_FILENAME'],
            recursive=True)
        if files:
            env['CM_DATASET_PREPROCESSED_PATH'] = env['CM_IMAGENET_PREPROCESSED_PATH']
        else:
            return {'return': 1, 'error': 'No preprocessed images found in ' +
                    env['CM_IMAGENET_PREPROCESSED_PATH']}
    else:
        if env.get('CM_DATASET_REFERENCE_PREPROCESSOR', "0") == "1":
            print("Using MLCommons Inference source from '" +
                  env['CM_MLPERF_INFERENCE_SOURCE'] + "'")

        env['CM_DATASET_PREPROCESSED_PATH'] = os.getcwd()
        if env['CM_DATASET_TYPE'] == "validation" and not exists(
                os.path.join(env['CM_DATASET_PATH'], "val_map.txt")):
            shutil.copy(os.path.join(env['CM_DATASET_AUX_PATH'], "val.txt"), os.path.join(env['CM_DATASET_PATH'],
                                                                                          "val_map.txt"))

    preprocessed_path = env['CM_DATASET_PREPROCESSED_PATH']

    if env.get('CM_DATASET_TYPE', '') == "validation" and not exists(
            os.path.join(preprocessed_path, "val_map.txt")):
        shutil.copy(os.path.join(env['CM_DATASET_AUX_PATH'], "val.txt"),
                    os.path.join(preprocessed_path, "val_map.txt"))

    if env.get('CM_DATASET_TYPE', '') == "calibration":
        env['CM_DATASET_IMAGES_LIST'] = env['CM_MLPERF_IMAGENET_CALIBRATION_LIST_FILE_WITH_PATH']
        env['CM_DATASET_SIZE'] = 500

    if env.get('CM_DATASET_DATA_TYPE_INPUT', '') == '':
        env['CM_DATASET_DATA_TYPE_INPUT'] = env['CM_DATASET_DATA_TYPE']

    return {'return': 0}


def postprocess(i):

    env = i['env']

    # finalize path
    preprocessed_path = env['CM_DATASET_PREPROCESSED_PATH']
    preprocessed_images_list = []
    preprocessed_imagenames_list = []

    match_text = "/*." + env.get("CM_DATASET_PREPROCESSED_EXTENSION", "*")
    for filename in sorted(glob.glob(preprocessed_path + match_text)):
        preprocessed_images_list.append(filename)
        preprocessed_imagenames_list.append(os.path.basename(filename))
    with open("preprocessed_files.txt", "w") as f:
        f.write("\n".join(preprocessed_images_list))
    with open("preprocessed_filenames.txt", "w") as f:
        f.write("\n".join(preprocessed_imagenames_list))

    env['CM_DATASET_PREPROCESSED_IMAGES_LIST'] = os.path.join(
        os.getcwd(), "preprocessed_files.txt")
    env['CM_DATASET_PREPROCESSED_IMAGENAMES_LIST'] = os.path.join(
        os.getcwd(), "preprocessed_filenames.txt")

    return {'return': 0}



#!/usr/bin/env python3

import json
import time
import os
import shutil
import numpy as np


import torch
import torchvision.models as models

import imagenet_helper
from imagenet_helper import (
    load_preprocessed_batch,
    image_list,
    class_labels,
    BATCH_SIZE)

# Writing the results out:
#
RESULTS_DIR = os.getenv('CM_RESULTS_DIR')
FULL_REPORT = os.getenv(
    'CM_SILENT_MODE',
    '0') in (
        'NO',
        'no',
        'OFF',
        'off',
    '0')

# Processing by batches:
#
BATCH_COUNT = int(os.getenv('CM_BATCH_COUNT', 1))

# Enabling GPU if available and not disabled:
#
USE_CUDA = (os.getenv('USE_CUDA', '').strip() == 'yes')


labels_path = os.environ['CM_CAFFE_IMAGENET_SYNSET_WORDS_TXT']


def load_labels(labels_filepath):
    my_labels = []
    input_file = open(labels_filepath, 'r')
    for l in input_file:
        my_labels.append(l.strip())
    return my_labels


labels = load_labels(labels_path)


data_layout = os.environ['ML_MODEL_DATA_LAYOUT']


def main():
    global BATCH_SIZE
    global BATCH_COUNT

    setup_time_begin = time.time()

    bg_class_offset = 0

    # Cleanup results directory
    if os.path.isdir(RESULTS_DIR):
        shutil.rmtree(RESULTS_DIR)
    os.mkdir(RESULTS_DIR)

    # Load the [cached] Torch model
    path_to_model_pth = os.environ['CM_ML_MODEL_FILE_WITH_PATH']

    model = models.resnet50(pretrained=False)
    model.load_state_dict(torch.load(path_to_model_pth))

    model.eval()

    # move the model to GPU for speed if available
    if USE_CUDA:
        model.to('cuda')

    setup_time = time.time() - setup_time_begin

    # Run batched mode
    test_time_begin = time.time()
    image_index = 0
    total_load_time = 0
    total_classification_time = 0
    first_classification_time = 0
    images_loaded = 0

    image_path = os.environ.get('CM_INPUT', '')
    if image_path != '':

        normalize_data_bool = True
        subtract_mean_bool = False

        from PIL import Image

        def load_and_resize_image(image_filepath, height, width):
            pillow_img = Image.open(image_filepath).resize(
                (width, height))  # sic! The order of dimensions in resize is (W,H)

            input_data = np.float32(pillow_img)

            # Normalize
            if normalize_data_bool:
                input_data = input_data / 127.5 - 1.0

            # Subtract mean value
            if subtract_mean_bool:
                if len(given_channel_means):
                    input_data -= given_channel_means
                else:
                    input_data -= np.mean(input_data)

        #    print(np.array(pillow_img).shape)
            nhwc_data = np.expand_dims(input_data, axis=0)

            if data_layout == 'NHWC':
                # print(nhwc_data.shape)
                return nhwc_data
            else:
                nchw_data = nhwc_data.transpose(0, 3, 1, 2)
                # print(nchw_data.shape)
                return nchw_data

        BATCH_COUNT = 1

    for batch_index in range(BATCH_COUNT):
        batch_number = batch_index + 1
        if FULL_REPORT or (batch_number % 10 == 0):
            print("\nBatch {} of {}".format(batch_number, BATCH_COUNT))

        begin_time = time.time()

        if image_path == '':
            batch_data, image_index = load_preprocessed_batch(
                image_list, image_index)
        else:
            batch_data = load_and_resize_image(image_path, 224, 224)
            image_index = 1

        torch_batch = torch.from_numpy(batch_data)

        load_time = time.time() - begin_time
        total_load_time += load_time
        images_loaded += BATCH_SIZE
        if FULL_REPORT:
            print("Batch loaded in %fs" % (load_time))

        # Classify one batch
        begin_time = time.time()

        # move the input to GPU for speed if available
        if USE_CUDA:
            torch_batch = torch_batch.to('cuda')

        with torch.no_grad():
            batch_results = model(torch_batch)

        classification_time = time.time() - begin_time
        if FULL_REPORT:
            print("Batch classified in %fs" % (classification_time))

        total_classification_time += classification_time
        # Remember first batch prediction time
        if batch_index == 0:
            first_classification_time = classification_time

        # Process results
        for index_in_batch in range(BATCH_SIZE):
            # skipping the background class on the left (if present)
            softmax_vector = batch_results[index_in_batch][bg_class_offset:]
            global_index = batch_index * BATCH_SIZE + index_in_batch

            res_file = os.path.join(RESULTS_DIR, image_list[global_index])

            with open(res_file + '.txt', 'w') as f:
                for prob in softmax_vector:
                    f.write('{}\n'.format(prob))

            top5_indices = list(reversed(softmax_vector.argsort()))[:5]
            for class_idx in top5_indices:
                print(
                    "\t{}\t{}\t{}".format(
                        class_idx,
                        softmax_vector[class_idx],
                        labels[class_idx]))
            print("")

    test_time = time.time() - test_time_begin

    if BATCH_COUNT > 1:
        avg_classification_time = (
            total_classification_time - first_classification_time) / (images_loaded - BATCH_SIZE)
    else:
        avg_classification_time = total_classification_time / images_loaded

    avg_load_time = total_load_time / images_loaded

    # Store benchmarking results:
    output_dict = {
        'setup_time_s': setup_time,
        'test_time_s': test_time,
        'images_load_time_total_s': total_load_time,
        'images_load_time_avg_s': avg_load_time,
        'prediction_time_total_s': total_classification_time,
        'prediction_time_avg_s': avg_classification_time,

        'avg_time_ms': avg_classification_time * 1000,
        'avg_fps': 1.0 / avg_classification_time,
        'batch_time_ms': avg_classification_time * 1000 * BATCH_SIZE,
        'batch_size': BATCH_SIZE,
    }
    with open('tmp-ck-timer.json', 'w') as out_file:
        json.dump(output_dict, out_file, indent=4, sort_keys=True)


if __name__ == '__main__':
    main()
